// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Java Audio Client GUI                                            ####
// ####    - A graphical Java client for the RTP Audio Server            ####
// #### Version 1.00  --  February 23, 2001                              ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                                                                  ####
// ##########################################################################


import java.io.*;
import java.util.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.Timer;
import java.awt.*;
import java.awt.event.*;


import AudioClient;
import Information;
import ScanURL;


/**
  * This class is the java swing GUI for the java audio client.
  *
  * @short   Java audio client
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class AudioClientGUI extends JFrame
   implements ActionListener, AdjustmentListener, ItemListener
{
   private JMenuBar  menuBar           = new JMenuBar();
   private JMenu     menuDatei         = new JMenu("File");
   private JMenuItem menuItemsDatei[]  = {
      new JMenuItem("Quit")
   };
   private JMenu     menuStrg        = new JMenu("Control");
   private JMenuItem menuItemsStrg[] = {
      new JMenuItem("Play"),
      new JMenuItem("Stop")
   };
   private JMenu     menuHilfe        = new JMenu("Help");
   private JMenuItem menuItemsHilfe[] = {
      new JMenuItem("Information"),
      new JMenuItem("Help")
   };

   private JButton       buttonPlay  = new JButton("Play");
   private JButton       buttonStop  = new JButton("Stop");
   private JToggleButton buttonPause = new JToggleButton("||");
   private JButton       buttonInfo  = new JButton("Info");

   private String     playingURL          = null;   
   private JScrollBar scrollbarPosition   = new JScrollBar(JScrollBar.HORIZONTAL,0,1,0,1);
   private JLabel     labelSource         = new JLabel("Source URL: (gaffel:7500/Test.list)");
   private JTextField textfieldSource     = new JTextField("",128);
   private JLabel     labelStatus         = new JLabel("Copyright Thomas Dreibholz");
   
   private JLabel labelsStatus[] = {
      new JLabel("Server Address:"),
      new JLabel("?"),
      new JLabel("TOS/Flow Label:"),
      new JLabel("?"),
      new JLabel("Server SSRC:"),
      new JLabel("?"),
      new JLabel("Client Address:"),
      new JLabel("?"),
      new JLabel("Client SSRC:"),
      new JLabel("?"),
      new JLabel("Bytes Received:"),
      new JLabel("?"),
      new JLabel("Packets Received:"),
      new JLabel("?"),
      new JLabel("Packets Lost:"),
      new JLabel("?"),
      new JLabel("Interarrival Jitter:"),
      new JLabel("?"),
      new JLabel("Quality:"),
      new JLabel("?"),
      new JLabel("Encoding:"),
      new JLabel("?"),
   };
   
   private JCheckBox   checkboxStereo    = new JCheckBox("Stereo",true);
   private JLabel      labelIPv6         = new JLabel("Using IPv4");
   private ButtonGroup groupBits;
   private ButtonGroup groupSamplingRate;
   private ButtonGroup groupEncoding;
   private JLabel      counter           = new JLabel("0:00.00",JLabel.CENTER);
   private JLabel      title             = new JLabel("",JLabel.LEFT);
   private JLabel      artist            = new JLabel("",JLabel.LEFT);
   private JLabel      comment           = new JLabel("",JLabel.LEFT);
   private JComboBox   comboBoxBits      = new JComboBox();
   private JComboBox   comboBoxRate      = new JComboBox();
   private JComboBox   comboBoxEncoding  = new JComboBox();
   private Timer       timer;
   private AudioClient client;
   private boolean     sliderValueSet    = false;
   private int         sliderUpdateDelay = 0;
   private Information information       = new Information();

   private final int   displayUpdateInterval = 350;
   private final int   eofRepeatInterval     = 20000 / displayUpdateInterval;
   private int         eofRepeatDelay;
   private boolean     closed = false;
   private boolean     callExit;


   // ###### Constructor ####################################################
   /**
     * Constructor for a new AudioClientGUI.
     *
     * @param url URL to store in textfield.
     * @param outputDevice -1 for AudioDebug, 0 for AudioDevice.
     * @param exitOnQuit true, if AudioClientGUI should call System.exit() when "Quit" is clicked; false otherwise.
     */
   public AudioClientGUI(String url, int outputDevice, boolean exitOnQuit)
   {
      client = new AudioClient(outputDevice);
      callExit = exitOnQuit;

/*
      try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.mac.MacLookAndFeel");
      } catch(Exception e) {
         System.err.println("L&F Error: " + e);
      }
*/

      // ====== Initialize menus ============================================
      for(int i = 0;i < menuItemsDatei.length;i++) {
         menuDatei.add(menuItemsDatei[i]);
         menuItemsDatei[i].addActionListener(this);
      }
      menuBar.add(menuDatei);
      for(int i = 0;i < menuItemsStrg.length;i++) {
         menuStrg.add(menuItemsStrg[i]);
         menuItemsStrg[i].addActionListener(this);
      }
      menuBar.add(menuStrg);
      for(int i = 0;i < menuItemsHilfe.length;i++) {
         menuHilfe.add(menuItemsHilfe[i]);
         menuItemsHilfe[i].addActionListener(this);
      }
      menuBar.add(menuHilfe);
      setJMenuBar(menuBar);     


      // ====== Initialize control panel ====================================
      getContentPane().setLayout(new GridLayout(2,2));
      JPanel panelControlButtons = new JPanel();
      panelControlButtons.setLayout(new GridLayout(1,4));
      panelControlButtons.add(buttonPlay);
      panelControlButtons.add(buttonStop);
      panelControlButtons.add(buttonPause);
      panelControlButtons.add(buttonInfo);
      buttonPlay.addActionListener(this);
      buttonStop.addActionListener(this);
      buttonPause.addActionListener(this);
      buttonInfo.addActionListener(this);
      if(url == null) {
         if(client.getIPVersion() == 6)
            textfieldSource.setText("rtpa://ipv6-localhost:7500/Test.list");
         else
            textfieldSource.setText("rtpa://localhost:7500/Test.list");
      }
      else {
         textfieldSource.setText(url);
      }
      textfieldSource.addActionListener(this);
      
      JPanel panelControl = new JPanel();
      panelControl.setLayout(new GridLayout(5,1));
      panelControl.add(panelControlButtons);
      panelControl.add(scrollbarPosition);
      panelControl.add(labelSource);
      panelControl.add(textfieldSource);
      panelControl.add(labelStatus);
      scrollbarPosition.addAdjustmentListener(this);


      // ====== Initialize quality panel ====================================
      int index = 0;
      int bits = client.getValidBits(index++);
      while(bits != 0) {
         comboBoxBits.addItem(new String("" + bits + " Bit"));
         bits = client.getValidBits(index++);
      }
      comboBoxBits.setSelectedIndex(index - 2);
      comboBoxBits.addItemListener(this);

      index = 0;
      int rate = client.getValidSamplingRate(index++);
      while(rate != 0) {
         comboBoxRate.addItem(new String("" + rate + " Hz"));
         rate = client.getValidSamplingRate(index++);
      }
      comboBoxRate.setSelectedIndex(index - 2);
      comboBoxRate.addItemListener(this);

      for(index = 0;index < 64;index++) {
         String name = client.getEncodingName(index);
         if(name.equals("")) {
            break;
         }
         comboBoxEncoding.addItem(name);
      }
      comboBoxEncoding.setSelectedIndex(0);
      comboBoxEncoding.addItemListener(this);

      JPanel checkboxPanel = new JPanel();
      checkboxPanel.setLayout(new GridLayout(1,2));
      checkboxStereo.addActionListener(this);
      checkboxPanel.add(checkboxStereo);
      checkboxPanel.add(labelIPv6);

      JPanel panelQuality = new JPanel();
      panelQuality.setLayout(new GridLayout(4,1));
      panelQuality.add(checkboxPanel);
      panelQuality.add(comboBoxBits);
      panelQuality.add(comboBoxRate);
      panelQuality.add(comboBoxEncoding);


      // ====== Initialize frame counter panel ==============================
      JPanel panelCounter  = new JPanel();
      Font   fontTimesBold = new Font("times",Font.BOLD,40);
      Color  color1        = new Color((float)0.6,(float)0.6,(float)0.7);
      Color  color2        = new Color((float)0.7,(float)0.6,(float)0.6);
      panelCounter.setLayout(new GridLayout(1,1));
      panelCounter.setBackground(color1);
      counter.setFont(fontTimesBold);
      counter.setForeground(Color.yellow);
      panelCounter.add(counter);

      JPanel panelMediaInfo  = new JPanel();
      panelMediaInfo.setLayout(new BoxLayout(panelMediaInfo,BoxLayout.X_AXIS));
      JPanel panelMediaInfo1 = new JPanel();
      JPanel panelMediaInfo2 = new JPanel();
      panelMediaInfo1.setLayout(new GridLayout(3,1));
      panelMediaInfo2.setLayout(new GridLayout(3,1));
      panelMediaInfo1.setBackground(color2);
      panelMediaInfo2.setBackground(color2);
      JLabel titleLabel   = new JLabel("Title:",JLabel.LEFT);
      JLabel artistLabel  = new JLabel("Artist:",JLabel.LEFT);
      JLabel commentLabel = new JLabel("Comment:",JLabel.LEFT);
      title.setForeground(Color.yellow);
      artist.setForeground(Color.yellow);
      comment.setForeground(Color.green);
      titleLabel.setForeground(Color.yellow);
      artistLabel.setForeground(Color.yellow);
      commentLabel.setForeground(Color.green);
      panelMediaInfo1.add(titleLabel);
      panelMediaInfo2.add(title);
      panelMediaInfo1.add(artistLabel);
      panelMediaInfo2.add(artist);
      panelMediaInfo1.add(commentLabel);
      panelMediaInfo2.add(comment);
      panelMediaInfo.add(panelMediaInfo1);
      panelMediaInfo.add(panelMediaInfo2);

      JPanel panelInfo = new JPanel();
      panelInfo.setLayout(new GridLayout(2,1));
      panelInfo.add(panelCounter);
      panelInfo.add(panelMediaInfo);


      // ====== Initialize status panel =====================================
      JPanel panelStatus = new JPanel();
      panelStatus.setLayout(new BoxLayout(panelStatus,BoxLayout.X_AXIS));
      JPanel panelStatus1 = new JPanel();
      panelStatus1.setLayout(new GridLayout(labelsStatus.length / 2,1));
      for(int i = 0;i < labelsStatus.length;i+=2) { 
         panelStatus1.add(labelsStatus[i]);
      };
      JPanel panelStatus2 = new JPanel();
      panelStatus2.setLayout(new GridLayout(labelsStatus.length / 2,1));
      for(int i = 1;i < labelsStatus.length;i+=2) { 
         panelStatus2.add(labelsStatus[i]);
      };     
      panelStatus.add(panelStatus1);
      panelStatus.add(panelStatus2);

      if(client.getIPVersion() == 6)
         labelIPv6.setText("Using IPv6");

      // ====== Initialize window ===========================================
      getContentPane().add(panelInfo);
      getContentPane().add(panelStatus);
      getContentPane().add(panelControl);
      getContentPane().add(panelQuality);
      setTitle("Java RTP Audio Client");
      setSize(640,375);
      show();
      updateCounter(0);
      timer = new Timer(displayUpdateInterval,this);
      timer.start();
   }

   
   // ###### Update frame counters ##########################################
   /**
     * Update frame counter.
     *
     * @param position Current position to be displayed.
     */
   synchronized private void updateCounter(long position) {
      long seconds = position / AudioClient.PositionStepsPerSecond;
      String minutesString  = Long.toString(seconds / 60);
      String secondsString  = Long.toString(seconds % 60);
      String positionString = Long.toString((position / (AudioClient.PositionStepsPerSecond / 100)) % 100);
      // while(minutesString.length() < 2)  minutesString  = "0" + minutesString;
      while(secondsString.length() < 2)  secondsString  = "0" + secondsString;
      while(positionString.length() < 2) positionString = "0" + positionString;
      counter.setText(minutesString + ":" + secondsString + "." + positionString);
   }


   // ###### TimerEvent #####################################################
   /**
     * This function updates all information on the connection to the server.
     * Ist is called by the update thread.
     */
   synchronized private void timerEvent() {
      if(client.playing()) {
         int error = client.getErrorCode();
         if(error >= AudioClient.ME_UnrecoverableError) {
            stop();
            System.out.println("Connection terminated due to encoder error #" + 
                               error + "!");
            return;
         }

         // ====== Display receiver statistics ==============================         
         labelsStatus[1].setText(client.getServerAddressString());
         labelsStatus[5].setText("$" + Integer.toHexString(client.getServerSSRC(0)));
         labelsStatus[7].setText(client.getOurAddressString());
         labelsStatus[9].setText("$" + Integer.toHexString(client.getOurSSRC()));
         labelsStatus[11].setText("" + client.getBytesReceived(-1));
         labelsStatus[13].setText(Integer.toString((int)client.getPacketsReceived(-1)) +
                                  " (" + Integer.toString(client.getRawBytesPerSecond() / 1024) + "KB/s)");

         int layers = client.getLayers();
         String flString = "";
         String jString  = "";
         long lost = 0;
         for(int i = 0;i < layers;i++) {
            int fractionLost = (int)(client.getFractionLost(i) * 1000.0);
            lost += client.getPacketsLost(i);
            int jitter = (int)(client.getJitter(i) / 10.0);
            if(i > 0) {
               flString += " ";
            }
            flString += (double)(fractionLost / 10.0) + "%";
            jString += (jitter / 100.0) + " ";
         }
         labelsStatus[17].setText(jString + "[ms]");
         labelsStatus[15].setText(lost + " (" + flString + ")");

         String qualityString = client.getSamplingRate() + "/" +
                                client.getBits() + "/";        
         if(client.getChannels() == 2)
            qualityString += "Stereo";
         else
            qualityString += "Mono";
         labelsStatus[19].setText(qualityString);
         labelsStatus[21].setText(client.getEncoding());


         // ====== Update MediaInfo =========================================
         MediaInfo mediaInfo = new MediaInfo();
         client.getMediaInfo(mediaInfo);
         title.setText(mediaInfo.Title);
         artist.setText(mediaInfo.Artist);
         comment.setText(mediaInfo.Comment);


         // ====== Update counters ==========================================
         updateCounter(client.getPosition());
         if(error == AudioClient.ME_EOF) {
            if(!buttonPause.isSelected()) {
               eofRepeatDelay++;
               labelStatus.setText("End of media reached! Restart in " +
                                   (((eofRepeatInterval - eofRepeatDelay) * displayUpdateInterval) / 1000) +
                                   " s...");
               if(eofRepeatDelay >= eofRepeatInterval) {
                  eofRepeatDelay = 0;
                  client.setPosition(0);
                  sliderUpdateDelay = 4;
                  sliderValueSet = false;
               }
            }
            else {
               labelStatus.setText("End of media reached!");
            }
         }
         else if(client.getBytesReceived(-1) > 0) {
            eofRepeatDelay   = 0;

            // ====== Get traffic class(es) =================================
            String label = new String();
            for(int i = 0;i < layers;i++) {
               label = label + client.getNameForTrafficClass(client.getTrafficClass(i));
               if(i < layers - 1) {
                  label = label + ":";
               }
            }

            // ====== Get flow label ========================================
            int flowLabel    = client.getFlowLabel(0);
            if(flowLabel == 0) {              
               labelsStatus[3].setText(label);
            }
            else {
               String flowLabelString = Integer.toHexString(flowLabel);
               while(flowLabelString.length() < 5)
                  flowLabelString = "0" + flowLabelString;
               labelsStatus[3].setText(label + " / $" + flowLabelString);
            }

            labelStatus.setText("Receiving data...");
         }
         
         // ====== Update scrollbar's range =================================
         if(!scrollbarPosition.getValueIsAdjusting()) {
            if(sliderUpdateDelay > 0)
               sliderUpdateDelay--;
            if(sliderUpdateDelay == 0) {
               sliderValueSet = true;
               scrollbarPosition.setMinimum(0);
               sliderValueSet = true;
               scrollbarPosition.setMaximum((int)(client.getMaxPosition() / (AudioClient.PositionStepsPerSecond / 100)));
               sliderValueSet = true;
               scrollbarPosition.setValue((int)(client.getPosition() / (AudioClient.PositionStepsPerSecond / 100)));
            }
         }
      }
      else {
         for(int i = 1;i <= 21;i += 2) {
            labelsStatus[i].setText("N/A"); 
         }
      }
   }
   

   // ###### Handle button clicks and menu selections #######################
   /**
     * actionPerformed() implementation of ActionListener interface.
     */
   synchronized public void actionPerformed(ActionEvent e) {
      String arg = e.getActionCommand();
      if(e.getSource() == timer) timerEvent();
      else if((arg.equals("Play")) || (e.getSource() == textfieldSource)) play();
      else if(arg.equals("Stop")) stop();
      else if(arg.equals("Quit")) quit();
      else if(arg.equals("||")) pause();
      else if(arg.equals("Info")) information();
      else if(arg.equals("Information")) information();
      else if(arg.equals("Help")) help();
      else if(arg.equals("Stereo")) setChannels(checkboxStereo.isSelected() ? 2 : 1);
   }


   // ###### Handle button clicks and menu selections #######################
   /**
     * itemStateChanged() implementation of ItemListener interface.
     */
   synchronized public void itemStateChanged(ItemEvent e) {
      Object source = e.getSource();
      if(source == comboBoxRate) {
         setSamplingRate(client.getValidSamplingRate(comboBoxRate.getSelectedIndex()));
      }
      else if(source == comboBoxBits) {
         setBits(client.getValidBits(comboBoxBits.getSelectedIndex()));
      }
      else if(source == comboBoxEncoding) {
         setEncoding(comboBoxEncoding.getSelectedIndex());
      }
      else {
         System.err.println("JavaClientGUI::itemStateChanged() - Invalid event!");
      }
   }


   // ###### Handle scrollbar movements #####################################
   /**
     * adjustmentValueChanged of AdjustmentListener interface.
     */
   synchronized public void adjustmentValueChanged(AdjustmentEvent e) {
      if(!sliderValueSet) {
         client.setPosition((AudioClient.PositionStepsPerSecond / 100) * (long)scrollbarPosition.getValue());
         buttonPause.setSelected(false);
         sliderUpdateDelay = 4;
      }
      else
         sliderValueSet = false;
   }


   // ###### help ###########################################################
   /**
     * Display help window.
     */
   synchronized public void help() {
      information.show("Help",
         "Java Client help:\n\n" +
         "1. Enter URL into textfield. Examples:\n" +
         "   rtpa://ipv6-odin!7500/../Test1.list\n" +
         "   rtpa://ipv6-odin:7500/../Test1.list\n" +
         "   rtpa://phoenix/../Test1.list\n" +
         "   rtpa://[ipv6-phoenix]:7500/../Test1.list\n" +
         "   rtpa://12.34.56.78:7500/../Test1.list\n" +
         "   rtpa://[1234::4567:123]:7500/../Test1.list\n\n" +
         "2. Click on \"Play\" to start playing."
      );
   }


   // ###### Information ####################################################
   /**
     * Display information window.
     */
     
   
   synchronized public void information() {
      information.show("Information","Copyright 1999-2001","Thomas Dreibholz","Dreibholz@bigfoot.com");
   }


   // ###### Quit ###########################################################
   /**
     * Quit program.
     */
   synchronized public void quit() {
      if(!closed) {
         timer.stop();
         stop();
         client.finalize();
         dispose();
         closed = true;
      }
      if(callExit) {
         // Note: Due to Java 1.3 exit() bug, it is necessary to kill java
         //       sending SIGINT to current process.
         AudioClient.killJavaInterpreter();
         // System.exit(0);
      }
   }


   // ###### Check, if client is closed #####################################
   /**
     * Check, if client is closed.
     *
     * @return true, if client is closed; false otherwise.
     */
   public boolean isClosed() {
      return(closed);
   }


   // ###### Play ###########################################################
   /**
     * Start playing.
     *
     * @param location URL of media to play (e.g. rtpa://gaffel:7500/Test.list).
     */
   synchronized public void play() {
      if(closed)
         return;

      // ====== Check, if only a media change is required ===================
      if((client.playing()) && (playingURL != null)) {
         ScanURL newURL = new ScanURL();
         if(newURL.scanURL(textfieldSource.getText(),"") == true) {
            ScanURL oldURL = new ScanURL();
            if(oldURL.scanURL(playingURL,"") == true) {
               if((oldURL.getHost().equals(newURL.getHost())) &&
                  (oldURL.getProtocol().equals(newURL.getProtocol()))) {
                  if(!oldURL.getPath().equals(newURL.getPath())) {
                     client.change(newURL.getPath());
                     playingURL = textfieldSource.getText();
                     return;
                  }
                  else return;
               }
            }
         }
      }
      stop();

      // ====== Start playing ===============================================
      buttonPause.setSelected(false);
      ScanURL newURL = new ScanURL();
      if((newURL.scanURL(textfieldSource.getText(),"") == true) &&
         (newURL.getProtocol().equals("rtpa"))) {
         if(client.play(newURL.getHost(),newURL.getPath())) {
            playingURL = textfieldSource.getText();
            eofRepeatDelay = 0;
            labelsStatus[1].setText(client.getServerAddressString());
            labelsStatus[7].setText(client.getOurAddressString());
            labelsStatus[21].setText(client.getEncoding());
            labelStatus.setText("Waiting for data from server...");
         }
         else {
            labelStatus.setText("Unable to find server!");
         }
      }
      else {
         labelStatus.setText("Bad URL! Check location and try again!");
      }
   }


   // ###### Pause ##########################################################
   /**
     * Set pause.
     */
   synchronized public void pause() {
      if(!closed) {
         if(buttonPause.isSelected())
            client.setPause(true);
         else
            client.setPause(false);
      }
   }


   // ###### Stop ###########################################################
   /**
     * Stop playing.
     */
   synchronized public void stop() {
      if(!closed) {
         client.stop();
         sliderValueSet    = false;
         sliderUpdateDelay = 0;
         scrollbarPosition.setValue(0);
         scrollbarPosition.setMinimum(0);
         scrollbarPosition.setMaximum(0);
         updateCounter(0);
         title.setText("");
         artist.setText("");
         comment.setText("");
         labelStatus.setText("Ready!");
      }
   }


   // ###### Set sampling rate ##############################################
   /**
     * Set audio sampling rate.
     *
     * @param rate New audio sampling rate.
     */
   synchronized public void setSamplingRate(int rate) {
      if(!closed) {
         client.setSamplingRate(rate);
      }
   }


   // ###### Set encoding ###################################################
   /**
     * Set encoding by index in client's decoder repository.
     *
     * @param index Index in decoder repository.
     */
   synchronized public void setEncoding(int index) {
      if(!closed) {
         client.setEncoding(index);
      }
   }


   // ###### Set bits #######################################################
   /**
     * Set number of audio bits.
     *
     * @param bits New number of audio bits.
     */
   synchronized public void setBits(int bits) {
      if(!closed) {
         client.setBits(bits);
      }
   }


   // ###### Set channels ###################################################
   /**
     * Set number of audio channels
     *
     * @param channels New number of audio channels.
     */
   synchronized public void setChannels(int channels) {
      if(!closed) {
         client.setChannels(channels);
      }
   }


   // ###### Get transport info #############################################
   /**
     * Get TransportInfo for current setting.
     *
     * @param transportInfo TransportInfo.
     */
   synchronized public void getTransportInfo(TransportInfo transportInfo) {
      if(!closed) {
         client.getTransportInfo(transportInfo);
      }
   }


   // ###### Set bandwidth limit ############################################
   /**
     * Set bandwidth limit.
     *
     * @param bandwidthLimit Bandwidth limit.
     */
   synchronized public void setBandwidthLimit(int bandwidthLimit) {
      if(!closed)
         client.setBandwidthLimit(bandwidthLimit);
   }
}
