// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Information                                                      ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import java.io.*;
import java.util.*;
import java.lang.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/**
  * This class implements a simple information window.
  *
  * @short   Information
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class Information implements ActionListener
{
   private boolean showing = false;
   private JFrame  frame   = null;


   // ###### Constructor ####################################################
   /**
     * Constructor.
     */
   Information() {
   }


   // ###### show 3-row information window ##################################
   /**
     * Show 3-row information window.
     *
     * @param title Window title.
     * @param t1 String #1.
     * @param t2 String #2.
     * @param t3 String #3.
     */
   public void show(String title, String t1, String t2, String t3) {
      close();
 
      frame = new JFrame();
      frame.setTitle(title);
            
      JPanel panel = new JPanel();
      panel.setLayout(new GridLayout(3,1));
      JLabel label1 = new JLabel(t1,JLabel.CENTER);
      JLabel label2 = new JLabel(t2,JLabel.CENTER);
      JLabel label3 = new JLabel(t3,JLabel.CENTER);
      panel.add(label1);
      panel.add(label2);
      panel.add(label3);
      panel.setMinimumSize(new Dimension(300,100));
      frame.getContentPane().add("North",panel);
      
      JButton button = new JButton("Okay!");
      button.addActionListener(this);      
      frame.getContentPane().add("South",button);
      
      frame.setSize(frame.getContentPane().getMinimumSize());
      frame.show();     
      showing = true;
   }


   // ###### show information window with textfield #########################
   /**
     * Show information window with textfield.
     *
     * @param title Window title.
     * @param text Text for textfield.
     */
   public void show(String title, String text) {
      close();

      frame = new JFrame();
      frame.setTitle(title);
            
      JTextArea textArea = new JTextArea(text);
      textArea.setEditable(false);
      textArea.setMinimumSize(new Dimension(350,250));
      frame.getContentPane().add("North",textArea);
      
      JButton button = new JButton("Okay!");
      button.addActionListener(this);      
      frame.getContentPane().add("South",button);
            
      frame.setSize(frame.getContentPane().getMinimumSize());
      frame.show();
      showing = true;
   }


   // ###### Close window ###################################################
   public void close() {
      if(showing) {
         showing = false;
         frame.dispose();
         frame = null;
      }
   }


   // ###### ActionListener implementation ##################################
   /**
     * actionPerformed() of ActionListener interface
     */
   public void actionPerformed(ActionEvent e) {
      String arg = e.getActionCommand();
      if("Okay!".equals(arg)) {
         close();
      }
   }
}
