//########################################################
//##                                                    ##
//##                 QoS Management                     ##
//##                 ==============                     ##
//##                                                    ##
//## Flow                                               ##
//##                                                    ##
//## Version 1.0                                        ##
//## Date  : March 2000                                 ##
//## Author: Simon Vey (vey@cs.uni-bonn.de)             ##
//##                                                    ##
//########################################################

#ifndef FLOW_H
#define FLOW_H

#include "qosmanagerinterface.h"
#include "../synchronizable.h"
#include "../transportinfo.h"
#include "qosmanagerbase.h"
#include "transportinfo2.h"

namespace Coral {

/**
 * This class contains information about one flow managed by the
 * QoS-Manager. 
 *
 * @short   Flow
 * @author  Simon Vey (vey@cs.uni-bonn.de)
 * @version 1.0
 */

class Flow : public Synchronizable {
 public:
  /**
   * Constructor. Creates a new flow.
   *
   * @param _ti Information about the flow from the application
   * @param interface Interface for application callback.
   * @param _qm Interface for communication with the QoSManager.
   */
  Flow(TransportInfo2 _ti, QoSManagerInterface* interface, QoSManagerBase* _qm, card32* _delayOfClass, card32* _lossRateOfClass);

  //~Flow();

  /**
   * Scales down the flow. 
   */
  bool scaleDown(card32 sessionID);

  /**
   * Scales up the flow.
   */
  bool scaleUp(card32 sessionID);

  /**
   * Sets the bandwidth calculation of the flow.
   */
  void setBandwidthCalculation(card64 bw);

  /**
   * Interface for application-callback
   */
  QoSManagerInterface* Interface;        // interface for application-callback 

  /**
   * Interface for communication with the QoSManager.
   */
  QoSManagerBase* qm;

  /**
   * information about the flow
   */
  TransportInfo2 ti;            // information about the flow

  TransportInfo ti_alt;
  
  /**
   * flow identifier
   */
  card32        flowID;                  // flow identifier

  /**
   * dynamic weight of the flow
   */
  card32        dynWeight;               // dynamic weight of the flow

  /**
   * current level of the flow
   */
  card8         level;                   // current level of the flow

  /**
   * max. allowed level of the flow.
   * caltulated once every N seconds
   */
  card8         maxLevel;                // max. allowed level of the flow
                                         // caltulatet once every N seconds  

  /**
   * max. accaptable delay for the flow.
   */
  card32        maxDelay;

  /**
   * Bandwidth, wanted by the flow
   */
  card64        wanted_bw;

  /**
   * Current Bandwidth of the flow
   */
  card64        current_bw;

  /**
   * Bandwidth calculation of the flow, calculated by the session.
   */
  card64        bandwidthCalculation;

  /**
   * Systemtime of last down-scaling
   */
  card64        lastChanged;

  card32*       delayOfClass;
  card32*       lossRateOfClass;

  bool layerFitsInClass(card8 priorityClass, card64 bandwidth, card8 lossRate, card32 delay);

 private:
};

}
#include "flow.icc"
#endif
