#include "layeredstream.h"
#include "session.h"
#include "qosmanager.h"
#include <vector>

namespace Coral {

LayeredStream::LayeredStream(card32 ID, ExtendedTransportInfo ti,QoSManagerInterface* qmi, Session* session_)
  : session(session_), id(ID), interface(qmi)
{
  int i=0,j=0;

  currentLevel = 255;

  maxLevel = ti.QualityLevels;

  //wantedBandwidth = ti.WantedBytesPerSecond.getValue();

  ti_alt = ti;

  start_fps = ti.StartFramesPerSecond; 

  Level* newLevel;  

  Layer* newLayer;

  for(i=0; i < ti.QualityLevels; i++) {
    newLevel = new Level(this);

    newLevel->setLevelUp(ti.QualityLevel[i].LevelUp);
    newLevel->setLevelDown(ti.QualityLevel[i].LevelDown);
    newLevel->setQuality(ti.QualityLevel[i].Quality);
    newLevel->setDelay(ti.QualityLevel[i].MaxTransferDelay);
    newLevel->setFPSScale(ti.QualityLevel[i].FramesPerSecondScale / 65536.);
        
    if(ID==1) {
      newLevel->setDelay(10000);   // 10ms
    }
    

    // cout << "delay von anwendung: " << ti.QualityLevel[i].MaxTransferDelay << endl;

    for(j=0; j< ti.QualityLevel[i].QualityLayers; j++) {
      newLayer = new Layer(newLevel);
      newLayer->setFrameSize(ti.QualityLevel[i].QualityLayer[j].FrameSize);
      newLayer->setLossRate(ti.QualityLevel[i].QualityLayer[j].MaxLossRate);
      
      if(j==0) {
	newLayer->setLossRate(13); // ca. 5%
      }
      
      //cout << "level " << i << ", layer " << j << ", lossrate: " <<  (int)ti.QualityLevel[i].QualityLayer[j].MaxLossRate << endl;

      newLevel->addLayer(newLayer);
    }

    //cout << "layeredstream constructor: framesize of layer 1: " << ti.QualityLevel[22].QualityLayer[1].FrameSize << endl;
    //cout << "layeredstream constructor: framesize of layer 2: " << ti.QualityLevel[22].QualityLayer[2].FrameSize << endl;

    /*
    newLayer = new Layer(newLevel);
    newLayer->setFrameSize(ti.QualityLevel[i].ExtensionBytesPerSecond);
    newLayer->setLossRate(255);
    newLevel->addLayer(*newLayer);
    */
    level.push_back(newLevel);
    //cout << "stream: level pointer of level " << i << " - " << &level[i] << endl;
  }

  // cout << this <<" stream of level 0 " << level[0]->getStream() << endl;
 
}

LayeredStream::~LayeredStream()
{
  int i=0;
  for(i=0; i < (int)level.size(); i++ ) {
    delete(level[i]);
  }
}

bool LayeredStream::scaleDown()
{
  cout << "scale down" << endl;
  static card64 lastCallTime = 0;
  card64 time = getMicroTime();

  // do not scale down within 1/10 sec !! (multiple synchronous calls of setFractionLost)
  if(time - lastCallTime < 100000) {
    return false;
  }

  lastCallTime = time;

  //cout << "scale down from level " << (int) currentLevel << endl;  

  //card32 sessionID = session->getID();
  
  QoSManager* qm = session->getQoSManager();

  // cout << "scale down flow " << id << " of session " << sessionID << endl;

  // cout << "from level " << (int) currentLevel << endl;

  int i;

  if(currentLevel == 255) return false;

  //  card8 oldLevel = level;
  Level* oldLevelP = getLevel(currentLevel);

  card8 newLevel = oldLevelP->getLevelDown();

  Level* newLevelP;

  if(newLevel != 255) {
    newLevelP = getLevel(newLevel);
    if(newLevelP->getNumLayers() > oldLevelP->getNumLayers()) {
      cout << "Error: Scale down: Number of layers must not grow, when scale down !!!" << endl;
      return false;
    }
  } else {                          // new level = 255 ?
    return false;
  }

  // ===== remove old layers ============================================================
  vector<card8> prioClass((int)oldLevelP->getNumLayers());
  vector<card64> old_bw((int)oldLevelP->getNumLayers());

  for(i=0; i < (int) oldLevelP->getNumLayers(); i++ ) {
    prioClass[i] = oldLevelP->getLayer(i)->getPriorityClass();
    old_bw[i]        = oldLevelP->getLayer(i)->getBandwidth();

    qm->setFreeBandwidthOfClass(prioClass[i], qm->getFreeBandwidthOfClass(prioClass[i]) + old_bw[i]);
  }

  // ===== add new layers ===============================================================
  card64 new_bw=0;

  if(newLevel != 255) {
    for(i=0; i < (int) newLevelP->getNumLayers(); i++ ) {
      new_bw =  newLevelP->getLayer(i)->getBandwidth();
      if(new_bw > old_bw[i]) {
	cout << "Error: Scale down: Layer must not became bigger when scale down !!!" << endl;
	return false;
      }

      qm->setFreeBandwidthOfClass(prioClass[i], qm->getFreeBandwidthOfClass(prioClass[i]) - new_bw);

      newLevelP->getLayer(i)->setPriorityClass(prioClass[i]);     // keep priority classes of the old level
    }
  }

  currentLevel = newLevel;  
  /*
  for(int i=0; i < 16; i++) {
    cout << "Priority Class " << i << " :  "<< qm->freeInPrio[i] << endl;
  }
  */
  // cout << "new level " << (int) level << endl;

  // cout << "to level " << (int) currentLevel << endl;

  lastChanged = getMicroTime();

  return true;
}

bool LayeredStream::scaleUp()
{
  //card32 sessionID = session->getID();

  QoSManager* qm = session->getQoSManager();

  // cout << "scale up flow " << id << " of session " << sessionID << endl;
  // cout << endl <<  "Scale up flow " << flowID << endl;
  int i;                             // counter

  card8 oldLevel = currentLevel;              // to remember the current level
  card8 newLevel;                      // new quality level
  
  //cout << "old level: " << (int) oldLevel << endl;


  // set new level
  if(currentLevel == 255) {
    newLevel = 0;
  } else {
    newLevel = getLevel(currentLevel)->getLevelUp();
  }

  if(newLevel > maxLevel) {
    //cout << "1" << endl;
    return false;
  }

  //cout << "newLevel" << (int) newLevel << endl;
  // cout << this << " stream of level 0 " << level[0]->getStream() << endl;

  if(newLevel == 255) {
    //cout << "2" << endl;    
    return false;                                   // there is no next level
  }

  //cout << "new Level " << (int)newLevel << endl;
 
  Level* newLevelP = getLevel(newLevel);  // Pointer to the new level.
  Level* oldLevelP = NULL;

  if(oldLevel != 255) {                            
    oldLevelP = getLevel(oldLevel);       // Pointer to the old level.
  }


  card8 numLayers = newLevelP->getNumLayers();        // number of layers in new level


  // cout << "numLayers " << (int) numLayers << endl;

  vector<bool> layerFit(numLayers);                 // to remember, wich layers fit
  vector<card64> layer_bw(numLayers);               // bandwidth of each new layer

  card64 level_bw = 0;                              // bandwidth, needed by the new level

  for(i=0; i < numLayers; i++) {               
    layerFit[i] = false;
    layer_bw[i] = newLevelP->getLayer(i)->getBandwidth();
    level_bw += layer_bw[i];
    //cout << "Layer" << i << " : " << layer_bw[i] << endl;
  }

  // ===== remove layers of the old level from priority classes =====================
  
  card8  oldPrio;
  card64 bw=0;

  if( oldLevel != 255 ) {                                 // is there a old level ?
    for(i=0; i < (int) oldLevelP->getNumLayers(); i++) {
      oldPrio = oldLevelP->getLayer(i)->getPriorityClass();
      bw      = oldLevelP->getLayer(i)->getBandwidth();

      qm->setFreeBandwidthOfClass(oldPrio, qm->getFreeBandwidthOfClass(oldPrio) + bw);
    }
  }

  //  for(i=0; i < 16; i++) {
  //    cout << qm->freeInPrio[i] << endl;
  //  }

  /*
  // ===== search a priority class for each layer of the new level ==================
  
  for(i=0; i < numLayers; i++) {                        // for each layer ...
    for(j=qm->getNumPrios()-1; j >= 0; j--) {                // search a class.



      if( layerFitsInClass(j, layer_bw[i], newLevelP->layer[i].maxLossRate, newLevelP->MaxTransferDelay ) ) {


	//if( qm->freeInPrio[j] >= layer_bw[i] ) {          // fits layer i in class j ?
	layerFit[i] = true;                             // layer i fits !
	qm -> freeInPrio[j] -= layer_bw[i];             // allocate bandwidth
	newLevelP->layer[i].priorityClass = j;          // set layers priority class
	break;                                          // class found for layer i ! So break!
      }
    }
    if(!layerFit[i]) {                                  // no class found for layer i ?
      break;                                            // stop searching!
    }
  }

  // ===== test, whether all layers had found a class ==================================

  bool fit = true;
  
  for(i=0; i < numLayers; i++) {
    if(layerFit[i] == false) {
      fit = false;
      break;
    }
  }
  */

  bool fit = newLevelP->findClasses();

  //  cout << "fit? " << fit << endl; 

  // ===== remove new layers and add old layers, if not all new layers fit =============

  card64 old_bw;

  if(!fit) {
    /*
    // remove new layers from priority classes
    for(i=0; i < numLayers; i++) {
      if(layerFit[i]) {
	oldPrio = newLevelP->layer[i].priorityClass;
	qm -> freeInPrio[ oldPrio ] += layer_bw[i];
      } else {
	break;
      }
    }
    */

    // add old layers to priority classes
    if( oldLevel != 255 ) {
      for(i=0; i < (int) oldLevelP->getNumLayers() ; i++ ) {
	oldPrio = oldLevelP->getLayer(i)->getPriorityClass();
	old_bw = oldLevelP->getLayer(i)->getBandwidth();

	qm->setFreeBandwidthOfClass(oldPrio, qm->getFreeBandwidthOfClass(oldPrio) - old_bw);
      }
    }
    //cout << "3" << endl;
    
    return false;
  } else {                                    // all layers fit
    currentLevel = newLevel;
  }

  //cout << "new level: " << (int) newLevel << endl;

  /*
  for(i=0; i < numLayers; i++) {
    cout << "Priority class of layer " << i << " : " << (int) newLevelP -> layer[i].priorityClass << endl;
  }
  */
  //current_bw = level_bw;                       // set the new bandwidth of the flow
  return true;
}

bool LayeredStream::resetToOldLevel()
{
  card8 newLevel = levelToRemember;
  // card32 sessionID = session->getID();
  
  QoSManager* qm = session->getQoSManager();

  int i;

  //  card8 oldLevel = level;
  Level* oldLevelP = getLevel(currentLevel);

  Level* newLevelP;

  if(newLevel != 255) {
    newLevelP = getLevel(newLevel);
  } else {                          // new level = 255 ?
    return false;
  }

  // ===== remove old layers ============================================================
  // vector<card8> prioClass((int)oldLevelP->getNumLayers());
  vector<card64> old_bw((int)oldLevelP->getNumLayers());

  card8 prio;

  for(i=0; i < (int) oldLevelP->getNumLayers(); i++ ) {
    prio = oldLevelP->getLayer(i)->getPriorityClass();
    old_bw[i]        = oldLevelP->getLayer(i)->getBandwidth();

    qm->setFreeBandwidthOfClass(prio, qm->getFreeBandwidthOfClass(prio) + old_bw[i]);
  }

  // ===== add new layers ===============================================================
  card64 new_bw=0;

  vector<bool> layerFits(newLevelP->getNumLayers());
  bool allLayersFit = true;


  if(newLevel != 255) {
    for(i=0; i < (int) newLevelP->getNumLayers(); i++ ) {
      layerFits[i] = false;
      new_bw =  newLevelP->getLayer(i)->getBandwidth();

      prio = newLevelP->getLayer(i)->getPriorityClass();

      if(qm->getFreeBandwidthOfClass(prio) >= new_bw ) {
	layerFits[i]=true;
	qm->setFreeBandwidthOfClass(prio, qm->getFreeBandwidthOfClass(prio) - new_bw);
	newLevelP->getLayer(i)->setPriorityClass(prio);     // keep priority classes of the old level
      } else {
	allLayersFit = false;
	break;
      }
    }
  }


  if(!allLayersFit) {
    // remove new layers
    for(i=0; i < newLevelP->getNumLayers(); i++) {
      if(!layerFits[i]) {
	break;
      } else {
	prio = newLevelP->getLayer(i)->getPriorityClass();
	new_bw = newLevelP->getLayer(i)->getBandwidth();
	qm->setFreeBandwidthOfClass(prio, qm->getFreeBandwidthOfClass(prio) + new_bw);
	//cout << "class " <<(int) prio << ":" << qm->getFreeBandwidthOfClass(prio) << endl;
	
      }
    }

    // add old layers
    for(i=0; i < oldLevelP->getNumLayers(); i++) {
      prio = oldLevelP->getLayer(i)->getPriorityClass();
      // cout << "class " <<(int) prio << ":" << qm->getFreeBandwidthOfClass(prio) << endl;
      qm->setFreeBandwidthOfClass(prio, qm->getFreeBandwidthOfClass(prio) - old_bw[i]);
      // cout << "class " <<(int) prio << ":" << qm->getFreeBandwidthOfClass(prio) << endl;

    }

    return false;
  } else {
    currentLevel = newLevel;
    return true;
  }


  /*
  for(int i=0; i < 16; i++) {
    cout << "Priority Class " << i << " :  "<< qm->freeInPrio[i] << endl;
  }
  */
  // cout << "new level " << (int) level << endl;

  return false;
}

/*
void LayeredStream::correctPointer()
{
  int i=0,j=0;
  for(i=0; i < level.size(); i++) {
    level[i]->setStream(this);
    for(j=0; j < level[i]->getNumLayers(); j++) {
      level[i]->getLayer(j)->setLevel(level[i]);
    }
  }
}
*/

}
