#include "simulator.h"

namespace Coral {

Simulator::Simulator()
  : Thread()
{
  FEL.readInputFile("events.in");
  
  outfile.open("simulator.log");

  qm.outfile = &outfile;

  TransportInfo ti;
  Layer l;

  ti.start_fps = 1;
  ti.QualityLevels = 2;
  ti.bw_weight = 1;
  ti.WantedMaxTransferDelay = 1;

  ti.QualityLevel[0].LevelUp = 1;
  ti.QualityLevel[0].LevelDown = 255;
  ti.QualityLevel[0].fps_scale = 1;
  ti.QualityLevel[0].Quality = 50;

  l.frameSize = 50000;
  l.maxLossRate = 0;
  ti.QualityLevel[0].addLayer(l);
  
  l.frameSize = 100000;
  l.maxLossRate = 0;
  ti.QualityLevel[0].addLayer(l);

  ti.QualityLevel[1].LevelUp = 255;
  ti.QualityLevel[1].LevelDown = 0;
  ti.QualityLevel[1].fps_scale = 1;
  ti.QualityLevel[1].Quality = 70;

  l.frameSize = 50000;
  l.maxLossRate = 0;
  ti.QualityLevel[1].addLayer(l);

  l.frameSize = 100000;
  l.maxLossRate = 0;
  ti.QualityLevel[1].addLayer(l);

  l.frameSize = 150000;
  l.maxLossRate = 0;
  ti.QualityLevel[1].addLayer(l);

  type.push_back(ti);

  start();
}

void Simulator::readTypes() {
  ifstream infile("types.in");

  char buffer[255];
  int intArg;
  // double doubleArg;
  String s;

  readOneWord(&infile, buffer);

  if(s.stringCompare("type",buffer) == 0) {
    infile >> intArg;
    if(intArg != (int)type.size()) {
      cerr << "wrong type number" << endl;
    }
  }
}

void Simulator::run()
{
  FELElement elem;
  double doubleTime=0;
  card64 intTime;
  int delayTime;
  int arg,arg2,arg3;

  card64 startTime = getMicroTime();
  outfile << startTime / 1000 << " start" << endl;

  while(!FEL.empty()) {
    // get time
    elem = FEL.pop();

    // wait till next event
    doubleTime = elem.doubleArg;
    intTime = (int) (doubleTime * 1000000) + startTime;
    delayTime = intTime - getMicroTime();

    if(delayTime > 0) {
      delay(delayTime);
    }

    // get event-type
    elem = FEL.pop();

    
    switch (elem.event) {
    case NEW_STREAM :
      // get session number
      elem = FEL.pop();
      arg = elem.intArg;
      // get flow number
      elem = FEL.pop();
      arg2 = elem.intArg;
      // get flow type
      elem = FEL.pop();
      arg3 = elem.intArg;
      flowID[arg][arg2] = qm.newFlow(sessionID[arg],type[arg3],&qmi);
      outfile << getMicroTime() / 1000 << " new stream " << flowID[arg][arg2];
      outfile << " of type " << arg3;
      outfile << " in session " << sessionID[arg] << endl;
      qm.writeLogInfo();
      break;
    case CLOSE_STREAM :
      // get session number
      elem = FEL.pop();
      arg = elem.intArg;
      // get flow number
      elem = FEL.pop();
      arg2 = elem.intArg;
      outfile << getMicroTime() / 1000 << " close stream " << flowID[arg][arg2];
      outfile << " of session " << sessionID[arg] << endl;
      qm.closeFlow(sessionID[arg],flowID[arg][arg2]);
      qm.writeLogInfo();
      break;
    case NEW_SESSION :
      // get session number
      elem = FEL.pop();
      arg = elem.intArg;
      sessionID[arg] = qm.newSession();
      outfile << getMicroTime() / 1000 << " new session " << sessionID[arg] << endl;
      break;
    case CLOSE_SESSION :
      // get session number
      elem = FEL.pop();
      arg = elem.intArg;
      outfile << getMicroTime() / 1000 << " close session " << sessionID[arg] << endl;
      qm.closeSession(sessionID[arg]);
      break;
    case SET_SESSION_WEIGHT :
      // get session number
      elem = FEL.pop();
      arg = elem.intArg;
      // get session weight
      elem = FEL.pop();
      arg2 = elem.intArg;
      outfile << getMicroTime() / 1000 << " session " << sessionID[arg] << " : set weight to ";
      outfile << arg2 << endl; 
      qm.setSessionWeight(sessionID[arg], arg2);
      qm.writeLogInfo();
      break;
    default: 
      cerr  << "no valid event" << endl;
    }
  }
  cout << "simulation finished" << endl;
}

}
