// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Audio Mixer implementation                                       ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "audiomixer.h"


#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/soundcard.h>


namespace Coral {


// ###### Constructor #######################################################
AudioMixer::AudioMixer(int mixerChannel, const char* name)
{
   Channel = mixerChannel;
   Device = open(name,O_RDWR);
   if(Device < 0) {
      cerr << "WARNING: Unable to open AudioMixer " << name << "." << endl;
   }
}


// ###### Destructor ########################################################
AudioMixer::~AudioMixer()
{
   if(Device >= 0) {
      close(Device);
   }
}


// ###### Get volume ########################################################
bool AudioMixer::getVolume(card8& left, card8& right)
{
   int volume;
   if(ioctl(Device,MIXER_READ(Channel),&volume) >= 0) {
      left = (card8)(volume & 0x7f);
      right = (card8)((volume >> 8) & 0x7f);
      return(true);
   }
   return(false);
}


// ###### Set volume ########################################################
bool AudioMixer::setVolume(const card8 left, const card8 right)
{
   unsigned int l = left;
   unsigned int r = right;
   if(l > 100) l = 100;
   if(r > 100) r = 100;
   int volume = left + (r << 8);
   return(ioctl(Device,MIXER_WRITE(Channel),&volume) >= 0);
}


}
