// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Test Receiver                                                    ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "socket.h"
#include "internetaddress.h"
#include "thread.h"
#include "breakdetector.h"
#include "trafficclassvalues.h"


using namespace Coral;


class ReceiverThread : public Thread
{
   // ====== Constructor ====================================================
   public:
   ReceiverThread(Socket* socket);


   // ====== Private data ===================================================
   private:
   void run();


   Socket* ReceiverSocket;
   card64  Received;
};


// ###### Constructor #######################################################
ReceiverThread::ReceiverThread(Socket* socket)
   : Thread()
{
   ReceiverSocket = socket;
   Received       = 0;
}


// ###### timerEvent() implementation #######################################
void ReceiverThread::run()
{
   const cardinal packetSize = 4096;
   char data[packetSize];

   for(;;) {
      ReceiverSocket->receive((void*)&data,packetSize);
   }
}


// ###### Main program ######################################################
int main(int argc, char** argv)
{
   // ====== Get arguments ==================================================
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << " [Port] {-force-ipv4}" << endl;
      exit(1);
   }
   bool optForceIPv4 = false;
   for(cardinal i = 2;i < (cardinal)argc;i++) {
      if(!(strcmp(argv[i],"-force-ipv4"))) {
         optForceIPv4 = true;
      }                    
   }
   if(optForceIPv4) {
      if(InternetAddress::UseIPv6 == true) {
         InternetAddress::UseIPv6 = false;
         cerr << "NOTE: IPv6 support disabled!" << endl;
      }
   }


   // ====== Initialize socket ==============================================
   Socket          receiverSocket(Socket::IP,Socket::UDP);
   InternetAddress receiverAddress(atol(argv[1]));
   if(receiverSocket.bind(receiverAddress) == false) {
      cerr << "ERROR: Binding to " << receiverAddress << " failed!" << endl;
      exit(1);
   }
   InternetAddress socketAddress;
   receiverSocket.getSocketAddress(socketAddress);


   ReceiverThread receiver(&receiverSocket);
   receiver.start();
   installBreakDetector();


   // ====== Print information ==============================================
   cout << "Test Receiver - Copyright (C) 1999-2001 Thomas Dreibholz" << endl;
   cout << "--------------------------------------------------------" << endl;
   cout << endl;
   cout << "Receiver address: " << socketAddress << endl;
   cout << endl;


   // ====== Main loop ======================================================
   card64 oldValue = 0;
   card64 oldTS    = getMicroTime();
   while(!breakDetected()) {
      Thread::delay(1000000,true);
      
      const card64 received = receiverSocket.getBytesReceived();
      const card64 now  = getMicroTime();

      if(received != oldValue) {
         char str[512];
         char tcString[32];
         const card8 trafficClass = receiverSocket.getReceivedTrafficClass();
         const char* tc = TrafficClassValues::getNameForTrafficClass(trafficClass);
         if(tc != NULL) {
            strcpy((char*)&tcString,tc);
         }
         else {
            snprintf((char*)&tcString,sizeof(tcString),"$%02x",trafficClass);
         }

         snprintf((char*)&str,sizeof(str),
                 "[Received: %12Ld Bytes = %12Ld Bits]  [Rate: %9Ld Bytes/s = %9Ld Bits/s, %s]",
                 received, received * 8,
                 (received - oldValue) * 1000000 / (now - oldTS),
                 (received - oldValue) * 8 * 1000000 / (now - oldTS),
                 tcString);
         cout << str << endl;
      }

      oldValue = received;
      oldTS    = now;
   }
   

   // ====== Clean-up =======================================================
   receiver.stop();
   cout << "Terminated!" << endl;
   return(0);
}
