// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Traffic Class Values implementation                              ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "trafficclassvalues.h"


namespace Coral {


// ###### Traffic class values ##############################################
const card8 TrafficClassValues::TCValues[TrafficClassValues::MaxValues] = {
   46,10,12,14,18,20,22,26,28,30,34,36,38,40,42,0
};


// ###### Traffic class names ###############################################
const char* TrafficClassValues::TCNames[TrafficClassValues::MaxValues] = {
   "EF",
   "AF11","AF12","AF13",
   "AF21","AF22","AF23",
   "AF31","AF32","AF33",
   "AF41","AF42","AF43",
   "TD1", "TD2",
   "BE"
};


// ###### Get index of traffic class ########################################
cardinal TrafficClassValues::getIndexForTrafficClass(const card8 trafficClass)
{
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(TCValues[i] == trafficClass) {
         return(i);
      }
   }
   return(MaxValues - 1);
}


// ###### Get name of traffic class #########################################
const char* TrafficClassValues::getNameForTrafficClass(const card8 trafficClass)
{
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(TCValues[i] == trafficClass) {
         return(TCNames[i]);
      }
   }
   return(NULL);
}


// ###### Get traffic class for name ########################################
const card16 TrafficClassValues::getTrafficClassForName(const char* name)
{
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(!(strcasecmp(TCNames[i],name))) {
         return((card16)TCValues[i]);
      }
   }
   return(0xffff);
}


}
