package bonn.infoIV.v3d2.buffer;

import bonn.infoIV.v3d2.util.ByteArrayToIntArray;
import bonn.infoIV.v3d2.util.SmartByteArrayInputStream;
import bonn.infoIV.v3d2.util.SmartDataInputStream;
import bonn.infoIV.v3d2.rtp.RTPPacket;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;

import bonn.infoIV.v3d2.rtp.PayloadType;

/**
 * Der SyncBuffer3D ist ein (per Hand???) synchronisierter Buffer.
 * Welcher Art der Buffer sein wird ist noch nicht klar (ob Byte, oder JavaObject).
 * WELCHE MOEGLICHKEITEN EROEFFNEN DIE STREAMS IN BEZUG AUF BYTE-ARRAY, BZW. OBJECT-ARRAY???
 * Wenn es keine gibt, so sollte diese Klasse (spaeter) ein Interface werden,
 * oder fuer die spezielle Anwendung abgeleitet werden! 
 */
public class SyncBuffer3D extends SyncBuffer
{
  public static final int PACKET_SIZE_BASE   = 1024; // 85 Knoten *3 Werte *4 Bytes = 1020 Bytes
  public static final int PACKET_SIZE_SPLITS = 1012; // 42 Splits *6 Werte *4 Bytes = 1008 Bytes

  public static final int SIZE_COUNT  = 3; // vertices, triangles, vertexSplits
  public static final int ARRAY_COUNT = 2; // baseMesh, vertexSplits

  protected boolean FirstVertexPacket_      = true;
  protected boolean FirstTrianglePacket_    = true;
  protected boolean FirstVertexSplitPacket_ = true;

  protected int ActualBaseTriangles_ = 0; // actual triangle-count of the frame been currently received
  protected int ActualBaseVertices_  = 0; // actual vertices-count of the frame been currently received
  protected int ActualVertexSplits_  = 0; // actual vertexSplits-count (max) of the frame been currently received

  private Object3D obj = null;
  private int[] Position_ = null;  // THE_NEW_SENDER

  protected SmartDataInputStream      DataIn_    = null; 
  protected SmartByteArrayInputStream ByteIn_    = null;
  protected byte[]                    ByteArray_ = null;
  protected ByteArrayToIntArray       Batia_     = null; 

  public SyncBuffer3D()
  {
    super();

    obj = (Object3D)(BufferedObjects_.elementAt(StreamIds_.indexOf(new Integer(OBJECT))));
    BiggestArraySize_ = obj.getArraySize(Object3D.VERTEX_SPLITS);

    ByteArray_ = new byte[RTPPacket.MAX_PAYLOAD_SIZE];
    ByteIn_    = new SmartByteArrayInputStream(ByteArray_);
    DataIn_    = new SmartDataInputStream(ByteIn_);
    Batia_     = new ByteArrayToIntArray(DataIn_);
    Position_  = new int[1];
  }


  protected Object3D getObject(Integer id)
  {
    int i = StreamIds_.indexOf(id);
    if(i>=0) return (Object3D)(BufferedObjects_.elementAt(i));
    else
    {
      BufferedObjects_.addElement(new Object3D(SIZE_COUNT, ARRAY_COUNT, id.intValue()));
      StreamIds_.addElement(id);
      return (Object3D)(BufferedObjects_.lastElement());
    }
  }


  protected void verifyObject(Integer id)
  {
    int i = StreamIds_.indexOf(id);
    if(i>=0) return;
    else
    {
      BufferedObjects_.addElement(new Object3D(SIZE_COUNT, ARRAY_COUNT, id.intValue()));
      StreamIds_.addElement(id);
    }
  }


  public void putMaxInfo(byte[] data, int identifier)
  {
    int[] maxInfo = new int[DoubleBufferedObject.MAX_INFO_SIZE];

    DataIn_.setNewBuffer(data);
    Batia_.setNewInputStream(DataIn_);
    try { Batia_.readInt(maxInfo); } 
    catch (IOException e) { System.out.println("Exception putting PM_MAX_INFO: " +e.getMessage()); }      
    obj = getObject(new Integer(identifier));
    obj.setMaxInfo(maxInfo);
  }


  public int[] getMaxInfo(int identifier)
  {
    obj = getObject(new Integer(identifier));
    return obj.getMaxInfo();
  }



  /**
   * putInfo.
   */
  public void putInfo(byte[] data, boolean  frame, int identifier)
  {
    int[] sizes = new int[SIZE_COUNT];

    try{ DataIn_.reset(); }
    catch(IOException e){ System.out.println("Kill'em, kill'em all!!!");}

    DataIn_.setNewBuffer(data);
    Batia_.setNewInputStream(DataIn_);

    try 
    { 
      Batia_.readInt(Position_);
      Batia_.readInt(sizes); 
    } 
    catch (IOException e)
    { 
      System.out.println("Exception putting PM_INFO: " +e.getMessage()); 
      e.printStackTrace();
      System.exit(0);
    }      

    ActualBaseVertices_  = sizes[0];
    ActualBaseTriangles_ = sizes[1];
    ActualVertexSplits_  = sizes[2];

//      System.out.println("Geschrieben: Vertices: " +ActualBaseVertices_ 
//                        +" Triangles: " +ActualBaseTriangles_ 
//                        +" Splits: " +ActualVertexSplits_);

    obj = getObject(new Integer(identifier));
    obj.setInfo(frame, sizes);

    FirstVertexPacket_      = true;
    FirstTrianglePacket_    = true;
    FirstVertexSplitPacket_ = true;
  }


  public void putBaseMeshVertices(byte[] data, boolean frame, int identifier)
  {
    if(FirstVertexPacket_)
    {
      FirstVertexPacket_ = false;
      putToArray(Object3D.BASE_MESH, frame, data, true, 
                 ActualBaseVertices_*Object3D.BASE_ENTITY, 
                 identifier, Object3D.BASE_PACKET);
    }
    else putToArray(Object3D.BASE_MESH, frame, data, false, 
                    Object3D.NO_SIZE, 
                    identifier, Object3D.BASE_PACKET);
  }


  public void putBaseMeshTriangles(byte[] data, boolean frame, int identifier)
  {
    if(FirstTrianglePacket_)
    {
      FirstTrianglePacket_=false;
      putToArray(Object3D.BASE_MESH, frame, data, false, 
                 ActualBaseTriangles_*Object3D.BASE_ENTITY, 
                 identifier, Object3D.BASE_PACKET);
    }
    else putToArray(Object3D.BASE_MESH, frame, data, false, 
                    Object3D.NO_SIZE, 
                    identifier, Object3D.BASE_PACKET);
  }


  public void putVertexSplits(byte[] data, boolean frame, int identifier)
  {
    if(FirstVertexSplitPacket_)
    {
      FirstVertexSplitPacket_ = false;
      putToArray(Object3D.VERTEX_SPLITS, frame, data, true, 
                 ActualVertexSplits_*Object3D.SPLIT_ENTITY, 
                 identifier, Object3D.SPLIT_PACKET);
    }
    else putToArray(Object3D.VERTEX_SPLITS, frame, data, false, 
                    Object3D.NO_SIZE, 
                    identifier, Object3D.SPLIT_PACKET);
  }
}
