package bonn.infoIV.v3d2.rtp;

import bonn.infoIV.v3d2.buffer.SyncBuffer;



public abstract class DataChannel extends Thread implements Channel, BaseDataChannel
{
  /** RTPPacket_ is used for receiving */
  protected RTPPacket  RTPPacket_ = null;
  /** MyBuffer_ is a reference to the Buffer, where the data is buffered, according to it's structure */
  protected SyncBuffer MyBuffer_  = null;
  protected Thread     MyConsumer_ = null;

  /** MyControl_ is a reference to a control-object, computing RTCP messages */
  protected Control    MyControl_ = null;

  /** Ssrc_ is the Id of the sender source */
  public int Ssrc_             = 0;

  /** Received_ is the total amount of RTP-packets received */ 
  public int Received_         = 0;
  /** ReceivedPrior_ is the amount of received RTP-packets prior to the last RTCP-RR-message */
  public int ReceivedPrior_    = 0;
  /** ReceivedInterval_ is the amount of received RTP-packets since the last RTCP-RR-message */
  public int ReceivedInterval_ = 0;

  /** ExpectedPrior_ is the amount of the expected RTP-packets prior to the last RTCP-RR-message. 
      It is computed, using the sequence numbers of RTP-packets. */
  public int ExpectedPrior_    = 0;
  /** ExpectedInterval_ is the amount of the expected RTP-packets since the last RTCP-RR-message. 
      It is computed, using the sequence numbers of RTP-packets. */
  public int ExpectedInterval_ = 0;

  /**
   * Running_ is true as long, as Channel Threads are not finished.
   * You have to set Running_ to 'false' to end a Channel-Thread properly.
   */
  protected boolean Running_     = true;


  /**
   * This constructor instantiates the RTPPacket_ and MyBuffer_.
   *
   * @param buffer - the SyncBuffer for this RTP-receiver.
   */
  public DataChannel(SyncBuffer buffer)
  {
    super();
    RTPPacket_ = new RTPPacket();
    MyBuffer_   = buffer;
  }

  /**
   * Sets the Control-object for this Channel.
   *
   * @param c - the control-object for this RTP-receiverchannel.
   */
  public void setControl(Control c)
  {
    MyControl_ = c;
  }

  /**
   * This abstract class should be implemented as soon, as the transport-protocol is clear.
   */
  public abstract void receive();


  /**
   * This abstract class should be implemented as soon, as the transport-protocol is clear.
   *
   * @param packet - the RTP-packet to be send.
   */
  public abstract void send(RTPPacket packet);


  /**
   * This method receives until the variable Running_ is set to false. 
   */
  public void run()
  {
    while(Running_) receive();
  }


  public void shutdown()
  {
    Running_ = false;
  }


  public void setConsumer(Thread consumer) { MyConsumer_ = consumer; }

}

