package bonn.infoIV.v3d2.rtp;

import bonn.infoIV.v3d2.util.ByteArrayToIntArray;
import bonn.infoIV.v3d2.util.IntArrayToByteArray;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.EOFException;

public class RTCPReportBlock
{
  public static final int REPORT_BLOCK_LENGTH       = 6;

  private static final int SSRC                      = 0;
  private static final int FRACTION_AND_PACKETS_LOST = 1;
  private static final int HIGHEST_SEQ_NUM_RECEIVED  = 2;
  private static final int INTERARRIVAL_JITTER       = 3;
  private static final int LAST_SR                   = 4;
  private static final int DELAY_SINCE_LAST_SR       = 5;


  private int[] Report_ = null;


  public RTCPReportBlock(int ssrc)
  {
    super();
    Report_ = new int[REPORT_BLOCK_LENGTH];
    Report_[SSRC]                      = ssrc;
    Report_[FRACTION_AND_PACKETS_LOST] = 0;
    Report_[HIGHEST_SEQ_NUM_RECEIVED]  = 0;
    Report_[INTERARRIVAL_JITTER]       = 0;
    Report_[LAST_SR]                   = 0;
    Report_[DELAY_SINCE_LAST_SR]       = 0;
  }


  public RTCPReportBlock()
  {
    super();
    Report_ = new int[REPORT_BLOCK_LENGTH];
    Report_[SSRC]                      = 0;
    Report_[FRACTION_AND_PACKETS_LOST] = 0;
    Report_[HIGHEST_SEQ_NUM_RECEIVED]  = 0;
    Report_[INTERARRIVAL_JITTER]       = 0;
    Report_[LAST_SR]                   = 0;
    Report_[DELAY_SINCE_LAST_SR]       = 0;
  }

  public int[] getReportBlock() { return Report_; }

  public void setSSRC(int v) { Report_[SSRC] = v; }
  public int getSSRC() { return Report_[SSRC]; }

  private int pow(int exp) 
  { 
    int x = 1;
    for(int i=0 ; i<exp ; i++) x*=2;
    return x;
  }

  public void setLostFraction(byte v)
  { 
    int packetLost = Report_[FRACTION_AND_PACKETS_LOST];
    if(packetLost<0) packetLost-=Integer.MIN_VALUE;
    for(int i=30 ; i>23 ; i--)
      if(packetLost>=pow(i)) packetLost-=pow(i);

    Report_[FRACTION_AND_PACKETS_LOST] = v*pow(24);
    Report_[FRACTION_AND_PACKETS_LOST] += packetLost;
  }
  public byte getLostFraction()
  {
    return (byte)(Report_[FRACTION_AND_PACKETS_LOST]/pow(24));
  }


  public void setLostPackets(int v) 
  { 
    byte fractionLost = (byte)(Report_[FRACTION_AND_PACKETS_LOST]/pow(24));
    Report_[FRACTION_AND_PACKETS_LOST] = v; 
    Report_[FRACTION_AND_PACKETS_LOST] += fractionLost*pow(24);
  }
  public int getLostPackets()
  {
    int packetLost = Report_[FRACTION_AND_PACKETS_LOST];
    if(packetLost<0) packetLost-=Integer.MIN_VALUE;
    for(int i=30 ; i>23 ; i--)
      if(packetLost>=pow(i)) packetLost-=pow(i);
    return packetLost;
  }

  public void setHighestSeqNumber(int v) { Report_[HIGHEST_SEQ_NUM_RECEIVED] = v; }
  public int getHighestSeqNumber() { return Report_[HIGHEST_SEQ_NUM_RECEIVED]; }

  public void setInterarrivalJitter(int v) { Report_[INTERARRIVAL_JITTER] = v; }
  public int getInterarrivalJitter() { return Report_[INTERARRIVAL_JITTER]; }

  public void setLSR(int v) { Report_[LAST_SR] = v; }
  public int getLSR() { return Report_[LAST_SR]; }

  public void setDLSR(int v) { Report_[DELAY_SINCE_LAST_SR] = v; }
  public int getDLSR() { return Report_[DELAY_SINCE_LAST_SR]; }


  //-------------------- read and write RTPHeaders -----------------------------
  public void readReport(DataInputStream in)
  {
    ByteArrayToIntArray batia = new ByteArrayToIntArray(in);
    try
    {
      batia.readInt(Report_);
    }
    catch(Exception e)
    { 
      System.err.println("[ERROR]: reading RTCP report block: " +e.getMessage()); 
    }
  }

  public void writeReport(DataOutputStream out)
  {
    IntArrayToByteArray iatba = new IntArrayToByteArray(out);
    try
    {
      iatba.writeInt(Report_);
    }
    catch(IOException e)  { System.err.println("[ERROR]: IOErr writing RTCP report block: " +e.getMessage()); }
  }

}


