package mpi_graphics;

import java.io.*;
import javax.media.j3d.*;

public class Mesh_Geometry
{
    public static final int NO_TRIANGLE = -1;
    public static final int NO_EDGE_INFORMATION = -1;
    public static final int NO_K = -1;
    public static final int POS_NOT_CONTAINED = -1;
    public static final int POS_0 = 0;
    public static final int POS_1 = 1;
    public static final int POS_2 = 2;
    public static final int POS_0_BIT = 0;
    public static final int POS_1_BIT = 1<<30;
    public static final int POS_2_BIT = 2<<30;
    public static final int POS_BLEND = (1<<30)-1;

    private IndexedTriangleArray geometry_data;
    protected int num_vertices_max = -1;
    protected int num_triangles_max = -1;
    protected int num_vertices_act = -1;
    protected int num_triangles_act = -1;
    
    // Fuer Testzwecke
    protected float[] vertex_array;
    protected int[] index_array;
    protected int shade_modus;

// ************************************************************************************
// Mesh_Geometry
// ************************************************************************************
    public Mesh_Geometry()
    {
    }
    
// ************************************************************************************
// Mesh_Geometry
// ************************************************************************************
    public Mesh_Geometry(Mesh_Info info, Mesh_Info base_info, int shade_modus)
    {
	int i;
	
	this.shade_modus=shade_modus;
	
	num_vertices_max=info.get_vertex_count();
	num_triangles_max=info.get_triangle_count();
	num_vertices_act=base_info.get_vertex_count();
	num_triangles_act=base_info.get_triangle_count();
	
	if (shade_modus==ColoringAttributes.SHADE_GOURAUD)
	{
	    geometry_data = new IndexedTriangleArray(num_vertices_max,
                        TriangleArray.COORDINATES|TriangleArray.NORMALS,num_triangles_max*3);
	    geometry_data.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
	    geometry_data.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
	    geometry_data.setCapability(GeometryArray.ALLOW_NORMAL_READ);
	    geometry_data.setCapability(GeometryArray.ALLOW_NORMAL_WRITE);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_COORDINATE_INDEX_READ);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_COORDINATE_INDEX_WRITE);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_NORMAL_INDEX_READ);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_NORMAL_INDEX_WRITE);
	}
	else
	{
	    int vertex_dim;

	    if (num_vertices_max>num_triangles_max)
		vertex_dim=num_vertices_max;
	    else
		vertex_dim=num_triangles_max;
		
	    geometry_data = new IndexedTriangleArray(vertex_dim,
                        TriangleArray.COORDINATES|TriangleArray.NORMALS,num_triangles_max*3);
	    geometry_data.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
	    geometry_data.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
	    geometry_data.setCapability(GeometryArray.ALLOW_NORMAL_READ);
	    geometry_data.setCapability(GeometryArray.ALLOW_NORMAL_WRITE);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_COORDINATE_INDEX_READ);
	    geometry_data.setCapability(IndexedGeometryArray.ALLOW_COORDINATE_INDEX_WRITE);
	
	    int[] i_afterburner = new int[3*num_triangles_max];

	    for (i=0;i<num_triangles_max;i++)
	    {
		i_afterburner[3*i]=i;
		i_afterburner[3*i+1]=i;
		i_afterburner[3*i+2]=i;
	    }
	
	    geometry_data.setNormalIndices(0,i_afterburner);
	}
	
	// Zur Beschleunigung
	vertex_array=new float[3*num_vertices_max];
	index_array=new int[3*num_triangles_max];
    }
    
// ************************************************************************************
// load_mesh_array
// ************************************************************************************
    final public void load_mesh_array(Mesh_Geometry_Array mesh_array)
    {
	int i;
	int max_vertices_number;
	
	max_vertices_number=mesh_array.num_vertices_max;
	if (max_vertices_number>num_vertices_max)
	    max_vertices_number=num_vertices_max;
	
	geometry_data.setCoordinates(0,mesh_array.vertex_array,0,max_vertices_number);
	num_vertices_act=mesh_array.num_vertices_base;
	num_triangles_act=mesh_array.num_triangles_base;
	
	for (i=0;i<3*max_vertices_number;i++)
	    vertex_array[i]=mesh_array.vertex_array[i];
	    
	for (i=0;i<3*num_triangles_act;i++)
	    index_array[i]=mesh_array.index_array_base[i];	

	for (i=3*num_triangles_act;i<3*num_triangles_max;i++)
	    index_array[i]=0;	
    }
    
// ************************************************************************************
// copy_mesh_array
// ************************************************************************************
    final public void copy_mesh_array(int v_start, int v_end, Mesh_Geometry_Array mesh_array)
    {
	int i;
	
	for (i=3*(mesh_array.num_vertices_base+v_start);i<3*(mesh_array.num_vertices_base+v_end);i++)
	{
	    vertex_array[i]=mesh_array.vertex_array[i];
	}
    }
    
// ************************************************************************************
// redraw
// ************************************************************************************
    final public void redraw()
    {
	int index;
	
	index=geometry_data.getCoordinateIndex(0);
	geometry_data.setCoordinateIndex(0,index);
    }
        
// ************************************************************************************
// get_geometry_data
// ************************************************************************************
    final public IndexedTriangleArray get_geometry_data()
    {
	return geometry_data;
    }
    
// ************************************************************************************
// get_num_vertices_act
// ************************************************************************************
    final public int get_num_vertices_act()
    {
	return num_vertices_act;
    }
    
// ************************************************************************************
// get_num_triangles_act
// ************************************************************************************
    final public int get_num_triangles_act()
    {
	return num_triangles_act;
    }
    
// ************************************************************************************
// load_vertices
// ************************************************************************************
    final public void load_vertices()
    {
	geometry_data.setCoordinates(0,vertex_array);
    }
    
// ************************************************************************************
// set_vertex
// ************************************************************************************
    final public void set_vertex(int i, float[] vertex)
    {
	//geometry_data.setCoordinate(i,vertex);
	vertex_array[3*i+0]=vertex[0];
	vertex_array[3*i+1]=vertex[1];
	vertex_array[3*i+2]=vertex[2];
    }
    
// ************************************************************************************
// get_vertex
// ************************************************************************************
    final public void get_vertex(int i, float[] vertex)
    {
	//geometry_data.getCoordinate(i,vertex);
	vertex[0]=vertex_array[3*i+0];
	vertex[1]=vertex_array[3*i+1];
	vertex[2]=vertex_array[3*i+2];
    }
    
// ************************************************************************************
// load_vertex_indices
// ************************************************************************************
    final public void load_vertex_indices()
    {
	geometry_data.setCoordinateIndices(0,index_array);
	
	if (shade_modus==ColoringAttributes.SHADE_GOURAUD)
	    geometry_data.setNormalIndices(0,index_array);
    }
    
// ************************************************************************************
// set_vertex_index
// ************************************************************************************
    final public void set_vertex_index(int index, int v_number)
    {
	//geometry_data.setCoordinateIndex(index,v_number);
	index_array[index]=v_number;
    }
    
// ************************************************************************************
// get_vertex_index
// ************************************************************************************
    final public int get_vertex_index(int index)
    {
	//return geometry_data.getCoordinateIndex(index);
	return index_array[index];
    }
    
// ************************************************************************************
// set_vertex_index
// ************************************************************************************
    final public void set_vertex_index(int t_number, int pos, int v_number)
    {
	//geometry_data.setCoordinateIndex(3*t_number+pos,v_number);
	index_array[3*t_number+pos]=v_number;
    }
    
// ************************************************************************************
// get_vertex_index
// ************************************************************************************
    final public int get_vertex_index(int t_number, int pos)
    {
	//return geometry_data.getCoordinateIndex(3*t_number+pos);
        if(pos<0) System.out.println("ALLARM");
	return index_array[3*t_number+pos];
    }
 
// ************************************************************************************
// set_vertex_indices
// ************************************************************************************
    final public void set_vertex_indices(int t_number, int[] v_number)
    {
	//geometry_data.setCoordinateIndices(3*t_number,v_number);
	index_array[3*t_number+0]=v_number[0];
	index_array[3*t_number+1]=v_number[1];
	index_array[3*t_number+2]=v_number[2];
    }
 
// ************************************************************************************
// get_vertex_indices
// ************************************************************************************
    final public void get_vertex_indices(int t_number, int[] v_number)
    {
	//geometry_data.getCoordinateIndices(3*t_number,v_number);
	v_number[0]=index_array[3*t_number+0];
	v_number[1]=index_array[3*t_number+1];
	v_number[2]=index_array[3*t_number+2];
    }

// ************************************************************************************
// set_normal
// ************************************************************************************
    final public void set_normal(int i, float[] normal)
    {
	geometry_data.setNormal(i,normal);
    }

// ************************************************************************************
// set_normals
// ************************************************************************************
    final public void set_normals(float[] normals)
    {
	geometry_data.setNormals(0,normals);
    }

// ************************************************************************************
// check_edge
// ************************************************************************************
    public boolean check_edge(int t_number, int v1, int v2)
    {
	int act_pos;

	act_pos=get_vertex_position(t_number,v1);
	act_pos=(act_pos+1)%3;

	if (get_vertex_index(t_number, act_pos)==v2)
	    return(true);

	return(false);
    }

// ************************************************************************************
// check_edge_changed_orientation
// ************************************************************************************
    public boolean check_edge_changed_orientation(int t_number, int v1, int v2)
    {
	int act_pos;

	act_pos=get_vertex_position(t_number,v1);
	act_pos=(act_pos+2)%3;

	if (get_vertex_index(t_number, act_pos)==v2)
	    return(true);

	return(false);
    }

// ************************************************************************************
// get_vertex_position
// ************************************************************************************
    final public int get_vertex_position(int t_number, int v_number)
    {
	int[] v_array = new int[3];
	
	//Global.Error_Occured("Teure Methode!!!");

	get_vertex_indices(t_number,v_array);
	
	if (v_array[0]==v_number)
		return(POS_0);

	if (v_array[1]==v_number)
		return(POS_1);

	if (v_array[2]==v_number)
		return(POS_2);

	System.out.println("Vertex Pos 0: " + v_array[0]);
	System.out.println("Vertex Pos 1: " + v_array[1]);
	System.out.println("Vertex Pos 2: " + v_array[2]);
	System.out.println("Dreieck: " + t_number);
	Global.Error_Occured(v_number + " ist kein Vertex von diesem Dreieck!!!");

	return(POS_NOT_CONTAINED);
    }
	
// ************************************************************************************
// set_act_mesh_info
// ************************************************************************************
    public void set_act_mesh_info(Mesh_Info info)
    {
	num_vertices_act=info.get_vertex_count();
	num_triangles_act=info.get_triangle_count();
    }
    
// ************************************************************************************
// write_obj_mesh
// ************************************************************************************
    public void write_obj_mesh(String filename) throws IOException
    {
	int i;
	String st;
	BufferedWriter out;
	float[] vertex = new float[3];
	int[] triangle_indices = new int[3];
	
	System.out.println("Erzeuge OBJ-Datei");

	out=new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename)));
	
	// Schreibe Vertices
	for (i=0;i<num_vertices_act;i++)
	    {
		get_vertex(i,vertex);
		out.write("v ",0,2);
		st=Float.toString(vertex[0]);
		out.write(st,0,st.length());
		out.write(" ",0,1);
		st=Float.toString(vertex[1]);
		out.write(st,0,st.length());
		out.write(" ",0,1);
		st=Float.toString(vertex[2]);
		out.write(st,0,st.length());
		out.newLine();
	    }
	// Schreibe Dreiecke
	for (i=0;i<num_triangles_act;i++)
	    {
		get_vertex_indices(i, triangle_indices);
		out.write("f ",0,2);
		st=Integer.toString(triangle_indices[0]+1);
		out.write(st,0,st.length());
		out.write(" ",0,1);
		st=Integer.toString(triangle_indices[1]+1);
		out.write(st,0,st.length());
		out.write(" ",0,1);
		st=Integer.toString(triangle_indices[2]+1);
		out.write(st,0,st.length());
		out.newLine();
	    }

	// Schliesse Datei
	out.close();
    }
}	
