package mpi_graphics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import java.util.*;

public class OFF_Container
{
    private Canvas3D c;
    private View act_view;
    private IndexedGeometryArray geometry_data;
    private Shape3D shape;
    protected PolygonAttributes polygon_attributes;
    private Point3d center_point = new Point3d();
    private int num_vertices_act = -1;
    private double radius;
    private final float grey = (float)0.5;
    protected JPanel contentPane = new JPanel();
    
// ************************************************************************************
// set_new_geometry
// ************************************************************************************
    final public void set_new_geometry(IndexedGeometryArray geometry_data)
    {
	this.geometry_data=geometry_data;
	shape.setGeometry(geometry_data);
    }
    
// ************************************************************************************
// get_content_pane
// ************************************************************************************
    final public JPanel get_content_pane()
    {
	return contentPane;
    }
    
// ************************************************************************************
// BranchGroup
// ************************************************************************************
    public BranchGroup createSceneGraph() {
	// Create the root of the branch graph
	BranchGroup objRoot = new BranchGroup();

	// Create the transform group node and initialize it to the
	// identity.  Enable the TRANSFORM_WRITE capability so that
	// our behavior code can modify it at runtime.  Add it to the
	// root of the subgraph.
	TransformGroup objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	objRoot.addChild(objTrans);
	
	Appearance ap = new Appearance();
	ap.setColoringAttributes(new ColoringAttributes(grey,grey,grey,ColoringAttributes.SHADE_GOURAUD));
	// SHADE_FLAT SHADE_GOURAUD
	polygon_attributes = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_BACK,0.0f,false);
	polygon_attributes.setCapability(PolygonAttributes.ALLOW_MODE_WRITE);
	polygon_attributes.setCapability(PolygonAttributes.ALLOW_CULL_FACE_WRITE );
	ap.setPolygonAttributes(polygon_attributes);
	// POLYGON_LINE  POLYGON_FILL
	// CULL_NONE CULL_BACK
	ap.setMaterial(new Material(new Color3f(0.2f, 0.2f, 0.2f),
				    new Color3f(0.1f, 0.1f, 0.1f),
                		    new Color3f(0.5f, 0.5f, 0.5f),
				    new Color3f(0.3f, 0.3f, 0.3f),
                		    24.0f));

	BoundingSphere bounds = get_object_bounds();

	// Create the transform group node to move the object center to zero
	TransformGroup objMiddle = new TransformGroup();
	Transform3D translation = new Transform3D();
	Vector3d dummy = new Vector3d(center_point);
	dummy.scale(-1.0);
	translation.set(dummy);
	objMiddle.setTransform(translation);
	shape=new Shape3D(geometry_data, ap);
	shape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
	objMiddle.addChild(shape);
	objTrans.addChild(objMiddle);

	  // Create the rotate behavior node
	  MouseRotate behavior = new MouseRotate();
	  behavior.setTransformGroup(objTrans);
	  objTrans.addChild(behavior);
	  behavior.setSchedulingBounds(bounds);

	  // Create the zoom behavior node
	  MouseZoom behavior2 = new MouseZoom();
	  behavior2.setTransformGroup(objTrans);
	  objTrans.addChild(behavior2);
	  behavior2.setFactor(1.0);
	  behavior2.setSchedulingBounds(bounds);

	  // Create the translate behavior node
	  MouseTranslate behavior3 = new MouseTranslate();
	  behavior3.setTransformGroup(objTrans);
	  objTrans.addChild(behavior3);
	  behavior3.setFactor(1.0);
	  behavior3.setSchedulingBounds(bounds);

        // Set up the background
        Color3f bgColor = new Color3f(0.0f, 0.0f, 0.1f);
        Background bgNode = new Background(bgColor);
        bgNode.setApplicationBounds(bounds);
        objRoot.addChild(bgNode);

        // Set up the ambient light
        Color3f ambientColor = new Color3f(0.1f, 0.1f, 0.1f);
        AmbientLight ambientLightNode = new AmbientLight(ambientColor);
        ambientLightNode.setInfluencingBounds(bounds);
        objRoot.addChild(ambientLightNode);

        // Set up the directional lights
        Color3f light1Color = new Color3f(1.0f, 1.0f, 0.9f);
        Vector3f light1Direction  = new Vector3f(4.0f, -7.0f, -12.0f);
        Color3f light2Color = new Color3f(0.3f, 0.3f, 0.4f);
        Vector3f light2Direction  = new Vector3f(-6.0f, -2.0f, -1.0f);

        DirectionalLight light1
            = new DirectionalLight(light1Color, light1Direction);
        light1.setInfluencingBounds(bounds);
        objRoot.addChild(light1);

        DirectionalLight light2
            = new DirectionalLight(light2Color, light2Direction);
        light2.setInfluencingBounds(bounds);
        objRoot.addChild(light2);

	return objRoot;
    }

// ************************************************************************************
// BoundingSphere
// ************************************************************************************
    public BoundingSphere get_object_bounds()
    {
	Point3d upper_bound_geometry_data = new Point3d();
	Point3d lower_bound_geometry_data = new Point3d();
	Point3d coord_act = new Point3d();
	int i;

	geometry_data.getCoordinate(0,upper_bound_geometry_data);
	geometry_data.getCoordinate(0,lower_bound_geometry_data);
	
	for (i=0;i<num_vertices_act;i++)
	{
	    geometry_data.getCoordinate(i,coord_act);
	    
	    if (coord_act.x>upper_bound_geometry_data.x)
		upper_bound_geometry_data.x=coord_act.x;
	    if (coord_act.y>upper_bound_geometry_data.y)
		upper_bound_geometry_data.y=coord_act.y;
	    if (coord_act.z>upper_bound_geometry_data.z)
		upper_bound_geometry_data.z=coord_act.z;
	    
	    if (coord_act.x<lower_bound_geometry_data.x)
		lower_bound_geometry_data.x=coord_act.x;
	    if (coord_act.y<lower_bound_geometry_data.y)
		lower_bound_geometry_data.y=coord_act.y;
	    if (coord_act.z<lower_bound_geometry_data.z)
		lower_bound_geometry_data.z=coord_act.z;
	}
	
	center_point.add(lower_bound_geometry_data,upper_bound_geometry_data);
	center_point.scale(0.5);
	radius=center_point.distance(upper_bound_geometry_data);
	return(new BoundingSphere(center_point,50.0*radius));
    }

// ************************************************************************************
// build_contentPane
// ************************************************************************************
    public void build_contentPane()
    {
	c = new Canvas3D(null) { 
	    Dimension prefSize = new Dimension(0, 0);
	    public Dimension getPreferredSize() {
		return prefSize;
		}
	};
	
        //Put everything in the content pane.
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
        contentPane.add("Center", c);
        contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	
	build_universe(c);
    }
    
// ************************************************************************************
// build_universe
// ************************************************************************************
    public void build_universe(Canvas3D c)
    {
	// Create a simple scene and attach it to the virtual universe
	BranchGroup scene = createSceneGraph();
	SimpleUniverse u = new SimpleUniverse(c);
       
	// Set up an initial view platform transform to pull back from
	// the objects so we can see them.
	TransformGroup vpTrans =
	    u.getViewingPlatform().getViewPlatformTransform();
	Transform3D translation = new Transform3D();
	translation.set( new Vector3d(0.0, 0.0, 3.0*radius) );
	vpTrans.setTransform(translation);
	
	act_view = u.getViewer().getView();
	act_view.setBackClipDistance(10*radius);
	act_view.setFrontClipDistance(0.1*radius);
	
	u.addBranchGraph(scene);
    }
    
// ************************************************************************************
// OFF_Container
// ************************************************************************************
    public OFF_Container(int num_vertices_act, IndexedGeometryArray geometry_data)
    {
    	this.num_vertices_act=num_vertices_act;
	this.geometry_data=geometry_data;
   }
}