#include <RTP/RTP_Session.hh>

void* rtp_send_thread(void* arg)
{
  RTP_Session* session = (RTP_Session*) arg;

  session->sendLoopRTP();

  return NULL;
}

void* rtp_recv_thread(void* arg)
{
  RTP_Session* session = (RTP_Session*) arg;

  session->recvLoopRTP();

  return NULL;
}

void* rtcp_send_thread(void* arg)
{
  RTP_Session* session = (RTP_Session*) arg;

  session->sendLoopRTCP();

  return NULL;
}

void* rtcp_recv_thread(void* arg)
{
  RTP_Session* session = (RTP_Session*) arg;

  session->recvLoopRTCP();

  return NULL;
}

pthread_t RTP_Session::transmit()
{
  pthread_attr_init(&rtp_send_thread_attr);
  pthread_attr_setscope(&rtp_send_thread_attr, PTHREAD_SCOPE_SYSTEM);
  pthread_create(&rtp_send_thread_id, &rtp_send_thread_attr, rtp_send_thread, this);

  pthread_attr_init(&rtcp_send_thread_attr);
  pthread_attr_setscope(&rtcp_send_thread_attr, PTHREAD_SCOPE_PROCESS);

  pthread_attr_init(&rtcp_recv_thread_attr);
  pthread_attr_setscope(&rtcp_recv_thread_attr, PTHREAD_SCOPE_PROCESS);

//pthread_create(&rtcp_send_thread_id, &rtcp_send_thread_attr, rtcp_send_thread, this);
  pthread_create(&rtcp_recv_thread_id, &rtcp_recv_thread_attr, rtcp_recv_thread, this);

  return rtp_send_thread_id;
}

void RTP_Session::finish() {}

pthread_t RTP_Session::join()
{
  pthread_attr_init(&rtp_recv_thread_attr);
  pthread_attr_setscope(&rtp_recv_thread_attr, PTHREAD_SCOPE_SYSTEM);
  pthread_create(&rtp_recv_thread_id, &rtp_recv_thread_attr, rtp_recv_thread, this);

  pthread_attr_init(&rtcp_send_thread_attr);
  pthread_attr_setscope(&rtcp_send_thread_attr, PTHREAD_SCOPE_PROCESS);

  pthread_attr_init(&rtcp_recv_thread_attr);
  pthread_attr_setscope(&rtcp_recv_thread_attr, PTHREAD_SCOPE_PROCESS);

  pthread_create(&rtcp_send_thread_id, &rtcp_send_thread_attr, rtcp_send_thread, this);
//pthread_create(&rtcp_recv_thread_id, &rtcp_recv_thread_attr, rtcp_recv_thread, this);

  return rtp_recv_thread_id;
}

void RTP_Session::leave() {}

void RTP_Session::susp()  {}

void RTP_Session::cont()  {}

RTP_Session::RTP_Session()
{
  // Init values
  initial  = true;
  we_sent  = false;
  shutdown = false;

  senders  = 0;
  members  = 1;
  pmembers = 1;

  rtcp_bw = 100;
  avg_rtcp_size = 50;

  // Init values
  ssrc        = 1;
  sequence_nr = 1;
}

// TD: Destruktor definiert, wg. Compiler-Warnung
RTP_Session::~RTP_Session()
{
}
// TD: Ende.

void RTP_Session::stop(RTCP_Bye* bye)
{
  /*
   * This function is responsible for deciding whether to
   * send an BYE packet now, or to reschedule transmission.
   * We use "unconditional reconsideration" to
   * reschedule the transmission of the BYE if necessary.
   */
  shutdown = true;

  gettimeofday(&tp, NULL);
  gettimeofday(&tc, NULL);

  tn = tc + transInterval();
  ts = tn - tc;

  while(1)
    {
      select(0, NULL, NULL, NULL, &ts);

      tn = tp + transInterval();
      if (tn <= tc.update())
	{          // TD: Keine TrafficClass hier?
	  send(bye,0);
	  return;
	}
      else ts = tn - tc;
    }
}

void RTP_Session::sendLoopRTCP()
{
  /*
   * This function is responsible for deciding whether
   * to send an RTCP report or to reschedule transmission.
   * It is also responsible for updating the pmembers, initial, tp,
   * and avg_rtcp_size state variables.
   */
  static int i=1;
  RTCP_Sender_Report RTCPSenderReport;
  RTCP_Receiver_Report RTCPReceiverReport;

  gettimeofday(&tp, NULL);
  gettimeofday(&tc, NULL);

  tn = tc + transInterval();
  ts = tn - tc;

  while(shutdown == false)
    {
      select(0, NULL, NULL, NULL, &ts);

      pmembers = members;
      tn = tp + transInterval();
      if (tn <= tc.update())
	{
	  initial = false;

	  cout << "Sending report...[" << i++ << "]" << endl;

	  if (we_sent == true)
	    {
	      // Fill sender report
	      RTCPSenderReport.rr[0].setSSRC(SSITable[ssrc].SSRC());
	      RTCPSenderReport.rr[0].setFractionLost(SSITable[ssrc].fractionLost());
	      RTCPSenderReport.rr[0].setPacketsLost(SSITable[ssrc].packetsLost());
	      RTCPSenderReport.rr[0].setLastSeqNum(SSITable[ssrc].lastSeqNum());
	      RTCPSenderReport.rr[0].setJitter(10);
	      RTCPSenderReport.rr[0].setLSR(20);
	      RTCPSenderReport.rr[0].setDLSR(30);

	      send(&RTCPSenderReport,0);
	    }
	  else
	    {
	      // Fill receiver report
	      RTCPReceiverReport.rr[0].setSSRC(SSITable[ssrc].SSRC());
	      RTCPReceiverReport.rr[0].setFractionLost(SSITable[ssrc].fractionLost());
	      RTCPReceiverReport.rr[0].setPacketsLost(SSITable[ssrc].packetsLost());
	      RTCPReceiverReport.rr[0].setLastSeqNum(SSITable[ssrc].lastSeqNum());
	      RTCPReceiverReport.rr[0].setJitter(10);
	      RTCPReceiverReport.rr[0].setLSR(20);
	      RTCPReceiverReport.rr[0].setDLSR(30);

	      cout << "RTCP-Packet\n";
	      cout << "{\n";
	      cout << "  RTCP_Header\n";
	      cout << "  {\n";
	      cout << "    Version     : " << RTCPReceiverReport.getVersion()    << "\n";
	      cout << "    Padding     : " << RTCPReceiverReport.getPadding()    << "\n";
	      cout << "    Count       : " << RTCPReceiverReport.getCount()      << "\n";
	      cout << "    Packet Type : " << RTCPReceiverReport.getPacketType() << "\n";
	      cout << "    Length      : " << RTCPReceiverReport.getLength()     << "\n";
	      cout << "    SSRC        : " << RTCPReceiverReport.getSSRC()       << "\n";
	      cout << "  }\n";
	      cout << "  RTCP_Payload\n";
	      cout << "  {\n";
	      cout << "    RTCP_Reception_Report_Block\n";
	      cout << "    {\n";
	      cout << "      SSRC            : " << RTCPReceiverReport.rr[0].getSSRC()         << "\n";
	      cout << "      Fraction Lost   : " << RTCPReceiverReport.rr[0].getFractionLost() << "\n";
	      cout << "      Packets Lost    : " << RTCPReceiverReport.rr[0].getPacketsLost()  << "\n";
	      cout << "      Last Seq Number : " << RTCPReceiverReport.rr[0].getLastSeqNum()   << "\n";
	      cout << "      Interar. Jitter : " << RTCPReceiverReport.rr[0].getJitter()       << "\n";
	      cout << "      LSR             : " << RTCPReceiverReport.rr[0].getLSR()          << "\n";
	      cout << "      DLSR            : " << RTCPReceiverReport.rr[0].getDLSR()         << "\n";
	      cout << "    }\n";
	      cout << "  }\n";
	      cout << "}\n";
	      cout << flush;

	      send(&RTCPReceiverReport,0);
	    }

	  /*
	   * We must redraw the interval. Don't reuse the
	   * one computed above, since its not actually
	   * distributed the same, as we are conditioned
	   * on it being small enough to cause a packet to
	   * be sent.
	   */
	  tp = tc;
	  tn = tc + transInterval();
	  ts = tn - tc;
	}
      else ts = tn - tc;
    }
}

void RTP_Session::recvLoopRTCP()
{
  RTCP_Union_Packet RTCPUnionPacket;

  while(shutdown == false)
    {
      unsigned short bytes = recv(&RTCPUnionPacket);
      switch(RTCPUnionPacket.getPacketType())
	{
	case RTCP_SR   : feedback(&RTCPUnionPacket);
	                 break;

	case RTCP_RR   : feedback(&RTCPUnionPacket);
	                 break;

	case RTCP_SDES : break;

	case RTCP_BYE  : cout << "Received Bye !" << endl;
	                 break;

	case RTCP_APP  : feedback(&RTCPUnionPacket);
	                 break;

	default        : cerr << "Unknown RTCP Packet Type: Packet ignored !" << endl;
	}

      RTCPUnionPacket.setLength((sizeof(RTCP_Union_Packet) / 4) - 1);
    }
}

unsigned short RTP_Session::send(RTCP_Packet* RTCPPacket, int trafficClass)
{
  RTCPSocket.setTrafficClass(trafficClass);
  unsigned short sentPacketSize = RTCPSocket.sent(RTCPPacket, (RTCPPacket->getLength() + 1) * 4);
  RTCPSocket.setTrafficClass(0);

  if (sentPacketSize > 0)
    avg_rtcp_size = (1.0/16.0) * sentPacketSize + (15.0/16.0) * avg_rtcp_size;

  return sentPacketSize;
}

unsigned short RTP_Session::recv(RTCP_Packet* RTCPPacket)
{
  int receivedPacketSize = RTCPSocket.recw(RTCPPacket, (RTCPPacket->getLength() + 1) * 4);
  avg_rtcp_size = (1.0/16.0) * receivedPacketSize + (15.0/16.0) * avg_rtcp_size;

  return receivedPacketSize;
}

double RTP_Session::transInterval()
{
  /*
   * Minimum average time between RTCP packets from this site (in
   * seconds).  This time prevents the reports from `clumping' when
   * sessions are small and the law of large numbers isn't helping
   * to smooth out the traffic.  It also keeps the report interval
   * from becoming ridiculously small during transient outages like
   * a network partition.
   */
  double const RTCP_MIN_TIME = 5.0;

  /*
   * Fraction of the RTCP bandwidth to be shared among active
   * senders.  (This fraction was chosen so that in a typical
   * session with one or two active senders, the computed report
   * time would be roughly equal to the minimum report time so that
   * we don't unnecessarily slow down receiver reports.) The
   * receiver fraction must be 1 - the sender fraction.
   */
  double const RTCP_SENDER_BW_FRACTION = 0.25;
  double const RTCP_RECVER_BW_FRACTION = (1 - RTCP_SENDER_BW_FRACTION);

  /*
   * To compensate for "unconditional reconsideration" converging to a
   * value below the intended average.
   */
  double const COMPENSATION = 2.71828 - 1.5;
  double rtcp_min_time = RTCP_MIN_TIME;

  /*
   * Very first call at application start-up uses half the min
   * delay for quicker notification while still allowing some time
   * before reporting for randomization and to learn about other
   * sources so the report interval will converge to the correct
   * interval more quickly.
   */
  if (initial)
    {
      rtcp_min_time = rtcp_min_time / 2;
    }

  /*
   * If there were active senders, give them at least a minimum
   * share of the RTCP bandwidth.  Otherwise all participants share
   * the RTCP bandwidth equally.
   */
  int n = members;            /* no. of members for computation */

  if (senders > 0 && senders < members * RTCP_SENDER_BW_FRACTION)
    {
      if (we_sent)
	{
	  rtcp_bw = rtcp_bw * RTCP_SENDER_BW_FRACTION;
	  n = senders;
	}
      else
	{
	  rtcp_bw = rtcp_bw * RTCP_RECVER_BW_FRACTION;
	  n = n - senders;
	}
    }

  /*
   * The effective number of sites times the average packet size is
   * the total number of octets sent when each site sends a report.
   * Dividing this by the effective bandwidth gives the time
   * interval over which those packets must be sent in order to
   * meet the bandwidth target, with a minimum enforced.  In that
   * time interval we send one report so this time is also our
   * average time between reports.
   */
  double t = avg_rtcp_size * n / rtcp_bw;            /* interval */
  if (t < rtcp_min_time) t = rtcp_min_time;

  /*
   * To avoid traffic bursts from unintended synchronization with
   * other sites, we then pick our actual next report interval as a
   * random number uniformly distributed between 0.5*t and 1.5*t.
   */
  t = t * (drand48() + 0.5);
  t = t / COMPENSATION;

  return t;
}

unsigned short RTP_Session::send(RTP_Packet* RTPPacket, unsigned short len, int trafficClass)
{
  RTPPacket->setSSRC(ssrc);
  RTPPacket->setSequenceNum(sequence_nr);

//unsigned short bytes = encode(RTPPacket, len);

  RTPSocket.setTrafficClass(trafficClass);
  unsigned short bytes = RTPSocket.sent(RTPPacket, len);
  RTPSocket.setTrafficClass(0);

  if (bytes == len)
    sequence_nr = sequence_nr + 1;
  else {
    // cerr << "sendRTP(): Transmission incomplete !" << endl;
  }

  return bytes;
}

unsigned short RTP_Session::recv(RTP_Packet* RTPPacket, unsigned short len)
{
//unsigned short bytes = decode(RTPPacket, len);

  unsigned short bytes = RTPSocket.recw(RTPPacket, len);

//if(NewMember(p))
//  {
//    AddMember(p);
//    members = members + 1;
//  };

  SSITable[RTPPacket->getSSRC()].update(RTPPacket->getSequenceNum());

  members = 1 + SSITable.size();

  return bytes;
}

/*
  if (TypeOfEvent(e) == RTCP_EVENT_REPORT)
    {
      if (NewSender(p) == false)
	{
	  RemoveSender(p);
	  senders = senders - 1;
	}
      if (NewMember(p) == false)
	{
	  RemoveMember(p);
	  members = members - 1;
	}
      if (members < pmembers)
	{
	  tn = tc + (((double) members)/pmembers) * (tn - tc);
	  tp = tc - (((double) members)/pmembers) * (tc - tp);
*/
	  /* Reschedule the next report for time tn */
/*	  Reschedule(tn, e);
	  pmembers = members;
	}
    }
  else
    {
      if (TypeOfEvent(e) == RTCP_EVENT_BYE) members = members + 1;
    }

if (NewMember(p))
{
  AddMember(p);
  members = members + 1;
}
*/
unsigned short RTP_Session::encode(RTP_Packet* RTPPacket, unsigned short len)
{
  static bool discarded = false;
  static bool encode = false;
  static char buffer[65516];
  static unsigned short i = 0;
  static unsigned short fec_seq = 1;
  static unsigned short fec_payload = 0;
  static FEC_Packet* FECPacket = (FEC_Packet*) buffer;

  unsigned short bytes;

  if ((RTPPacket->getPayloadType() >= 0) && (RTPPacket->getPayloadType() < 3))
    {
      /*
      if (i != 0) {
        RTPSocket.setTrafficClass(30);
	bytes = RTPSocket.sent(RTPPacket, len);
	RTPSocket.setTrafficClass(0);
      }
      else
	bytes = len;
      */
      if ((rand() % 2 == 0) && (discarded == false))
	{
	  bytes = len;
	  discarded = true;
	  //cout << "Packet " << RTPPacket->getSequenceNum() << " discarded !" << endl;
	}
      else
	{
          RTPSocket.setTrafficClass(30);
	  bytes = RTPSocket.sent(RTPPacket, len);
	  RTPSocket.setTrafficClass(0);
	  discarded = false;
	}
    }
  else {
      RTPSocket.setTrafficClass(30);
      bytes = RTPSocket.sent(RTPPacket, len);
      RTPSocket.setTrafficClass(0);
  }

  if ((RTPPacket->getPayloadType() >= 0) && (RTPPacket->getPayloadType() < 3))
    encode = true;

  if (encode == true)
    {
      XOR(FECPacket, RTPPacket, len);

      if (fec_payload < len - sizeof(RTP_Packet))
	fec_payload = len - sizeof(RTP_Packet);

      if (++i == 2)
	{
	  send(FECPacket, sizeof(FEC_Packet) + fec_payload, fec_seq++, RTPPacket->getTimestamp(), RTPPacket->getSequenceNum() - 1,0);
	  reset(FECPacket, sizeof(FEC_Packet) + fec_payload);
	  fec_payload = 0;
	  i = 0;

	  if (RTPPacket->getPayloadType() == 3)
	    encode = false;
	}
    }
  return bytes;
}

unsigned short RTP_Session::decode(RTP_Packet* RTPPacket, unsigned short len)
{
  static char bufferA[65516];
  static char bufferB[65516];
  static char bufferC[65516];

  static RTP_Packet  RTPHeader;
  static RTP_Packet* NXTPacket = NULL;
  static FEC_Packet* FECPacket = (FEC_Packet*) bufferA;
  static RTP_Packet* RECPacket = (RTP_Packet*) bufferB;
  static RTP_Packet* BUFPacket = (RTP_Packet*) bufferC;

  static unsigned short fec_len = 0;
  static unsigned short rec_len = 12;
  static unsigned short buf_len = 0;
  static unsigned short rtp_len = 0;
  static unsigned short nxt_len = 0;

  static unsigned short prv = 0;
  static unsigned short cur = 0;
  static unsigned short cup = 0;
  static unsigned short nxt = 0;

  static Time time;
  static bool wrapped = false;
  static FEC_State status = NO_RECOVER;

  if (status == RECOVERED)
    {
      prv = NXTPacket->getSequenceNum();
      transit(RTPPacket, NXTPacket, nxt_len);
      status = NO_RECOVER;
      return nxt_len;
    };

  rtp_len = RTPSocket.recw(RTPPacket, len);

  RTPSocket.recw(&RTPHeader, sizeof(RTPHeader), MSG_PEEK);

  cur = RTPPacket->getSequenceNum();
  cup = RTPPacket->getSequenceNum() - 1;
  nxt = RTPHeader.getSequenceNum();

  // Ignore FEC Packets from the receive buffer that we do not need
  if (prv > cur) wrapped = true;

  if (wrapped)
    {
      while ((FECSocket.selectRead(&time) == 1) && (FECPacket->getSNBase() > cup))
	{
	  fec_len = FECSocket.recw(FECPacket, 65516);
	  wrapped = false;
	}
    };

  while ((FECSocket.selectRead(&time) == 1) && (FECPacket->getSNBase() < cup))
    fec_len = FECSocket.recw(FECPacket, 65516);

  // Check if recovery is necessary and possible
  if ((prv + 1 != cur) && (FECPacket->getSNBase() == cup))
    status = RECOVER_BASE;
  else if ((cur != nxt - 1) && (FECPacket->getSNBase() == cur))
    status = RECOVER_EXT;
  else
    status = NO_RECOVER;

  // If necessary and possible, recover from error
  switch (status)
    {
    case RECOVER_BASE : reset(RECPacket, rec_len);
                        rec_len = reconstruct(RECPacket, RTPPacket, rtp_len, FECPacket, fec_len);
			transit(BUFPacket, RTPPacket, rtp_len);
			transit(RTPPacket, RECPacket, rec_len);
			buf_len = rtp_len;
			rtp_len = rec_len;
			NXTPacket = BUFPacket;
			nxt_len = buf_len;
			status = RECOVERED;
			break;

    case RECOVER_EXT  : reset(RECPacket, rec_len);
                        rec_len = reconstruct(RECPacket, RTPPacket, rtp_len, FECPacket, fec_len);
			NXTPacket = RECPacket;
			nxt_len = rec_len;
			status = RECOVERED;
			break;

    default           : break;
    }

  prv = RTPPacket->getSequenceNum();
  return rtp_len;
}

void RTP_Session::send(FEC_Packet* FECPacket, unsigned short len, unsigned short nr, unsigned int ts, unsigned short base, int trafficClass)
{
  FECPacket->setVersion(RTP_VERSION);
  FECPacket->setPayloadType(4);
  FECPacket->setSequenceNum(nr);
  FECPacket->setTimestamp(ts);
  FECPacket->setSSRC(ssrc);
  FECPacket->setSNBase(base);
  FECPacket->setMask(3);

  FECSocket.setTrafficClass(32);
  FECSocket.sent(FECPacket, len);
  FECSocket.setTrafficClass(0);
}

unsigned short RTP_Session::reconstruct(RTP_Packet* RECPacket, RTP_Packet* RTPPacket, unsigned short rtp_len, FEC_Packet* FECPacket, unsigned short fec_len)
{
  XOR(RECPacket, RTPPacket, rtp_len);
  XOR(RECPacket, FECPacket, fec_len);

  RECPacket->setVersion(RTPPacket->getVersion());

  if (FECPacket->getSNBase() == RTPPacket->getSequenceNum())
    RECPacket->setSequenceNum(RTPPacket->getSequenceNum() + 1);
  else
    RECPacket->setSequenceNum(RTPPacket->getSequenceNum() - 1);

  RECPacket->setSSRC(RTPPacket->getSSRC());

  unsigned short rec_len = ((rtp_len - sizeof(RTP_Packet)) ^ FECPacket->getLNRecovery()) + sizeof(RTP_Packet);

  return rec_len;
}

void RTP_Session::transit(RTP_Packet* dst, RTP_Packet* src, unsigned short len)
{
  dst->setVersion    ( src->getVersion()     );
  dst->setPadding    ( src->getPadding()     );
  dst->setExtension  ( src->getExtension()   );
  dst->setCSRCCount  ( src->getCSRCCount()   );
  dst->setMarker     ( src->getMarker()      );
  dst->setPayloadType( src->getPayloadType() );
  dst->setSequenceNum( src->getSequenceNum() );
  dst->setTimestamp  ( src->getTimestamp()   );
  dst->setSSRC       ( src->getSSRC()        );

  char* dst_data = ((char*) dst) + sizeof(RTP_Packet);
  char* src_data = ((char*) src) + sizeof(RTP_Packet);

  for (unsigned short j = 0; j < len - sizeof(RTP_Packet); j++)
    dst_data[j] = src_data[j];
}

void RTP_Session::XOR(FEC_Packet* FECPacket, RTP_Packet* RTPPacket, unsigned short len)
{
  FECPacket->setPadding    ( FECPacket->getPadding()    ^ RTPPacket->getPadding()     );
  FECPacket->setExtension  ( FECPacket->getExtension()  ^ RTPPacket->getExtension()   );
  FECPacket->setCSRCCount  ( FECPacket->getCSRCCount()  ^ RTPPacket->getCSRCCount()   );
  FECPacket->setMarker     ( FECPacket->getMarker()     ^ RTPPacket->getMarker()      );
  FECPacket->setPTRecovery ( FECPacket->getPTRecovery() ^ RTPPacket->getPayloadType() );
  FECPacket->setTSRecovery ( FECPacket->getTSRecovery() ^ RTPPacket->getTimestamp()   );
  FECPacket->setLNRecovery ( FECPacket->getLNRecovery() ^ (len - sizeof(RTP_Packet))  );

  char* fec_data = ((char*) FECPacket) + sizeof(FEC_Packet);
  char* rtp_data = ((char*) RTPPacket) + sizeof(RTP_Packet);

  for(unsigned short i = 0; i < len - sizeof(RTP_Packet); i++)
    fec_data[i] = fec_data[i] ^ rtp_data[i];
}

void RTP_Session::XOR(RTP_Packet* RECPacket, RTP_Packet* RTPPacket, unsigned short len)
{
  RECPacket->setPadding     ( RECPacket->getPadding()     ^ RTPPacket->getPadding()     );
  RECPacket->setExtension   ( RECPacket->getExtension()   ^ RTPPacket->getExtension()   );
  RECPacket->setCSRCCount   ( RECPacket->getCSRCCount()   ^ RTPPacket->getCSRCCount()   );
  RECPacket->setMarker      ( RECPacket->getMarker()      ^ RTPPacket->getMarker()      );
  RECPacket->setPayloadType ( RECPacket->getPayloadType() ^ RTPPacket->getPayloadType() );
  RECPacket->setTimestamp   ( RECPacket->getTimestamp()   ^ RTPPacket->getTimestamp()   );

  char* rec_data = ((char*) RECPacket) + sizeof(RTP_Packet);
  char* rtp_data = ((char*) RTPPacket) + sizeof(RTP_Packet);

  for(unsigned short i = 0; i < len - sizeof(RTP_Packet); i++)
    rec_data[i] = rec_data[i] ^ rtp_data[i];
}

void RTP_Session::XOR(RTP_Packet* RECPacket, FEC_Packet* FECPacket, unsigned short len)
{
  RECPacket->setPadding     ( RECPacket->getPadding()     ^ FECPacket->getPadding()    );
  RECPacket->setExtension   ( RECPacket->getExtension()   ^ FECPacket->getExtension()  );
  RECPacket->setCSRCCount   ( RECPacket->getCSRCCount()   ^ FECPacket->getCSRCCount()  );
  RECPacket->setMarker      ( RECPacket->getMarker()      ^ FECPacket->getMarker()     );
  RECPacket->setPayloadType ( RECPacket->getPayloadType() ^ FECPacket->getPTRecovery() );
  RECPacket->setTimestamp   ( RECPacket->getTimestamp()   ^ FECPacket->getTSRecovery() );

  char* rec_data = ((char*) RECPacket) + sizeof(RTP_Packet);
  char* fec_data = ((char*) FECPacket) + sizeof(FEC_Packet);

  for(unsigned short i = 0; i < len - sizeof(FEC_Packet); i++)
    rec_data[i] = rec_data[i] ^ fec_data[i];
}

void RTP_Session::reset(FEC_Packet* FECPacket, unsigned short len)
{
  FECPacket->setVersion(0);
  FECPacket->setPadding(0);
  FECPacket->setExtension(0);
  FECPacket->setCSRCCount(0);
  FECPacket->setMarker(0);
  FECPacket->setPayloadType(0);
  FECPacket->setSequenceNum(0);
  FECPacket->setTimestamp(0);
  FECPacket->setSSRC(0);
  FECPacket->setPTRecovery(0);
  FECPacket->setTSRecovery(0);
  FECPacket->setLNRecovery(0);

  char* fec_data = ((char*) FECPacket) + sizeof(FEC_Packet);

  for(unsigned short i = 0; i < len - sizeof(FEC_Packet); i++)
    fec_data[i] = 0;
}

void RTP_Session::reset(RTP_Packet* RECPacket, unsigned short len)
{
  RECPacket->setVersion(0);
  RECPacket->setPadding(0);
  RECPacket->setExtension(0);
  RECPacket->setCSRCCount(0);
  RECPacket->setMarker(0);
  RECPacket->setPayloadType(0);
  RECPacket->setSequenceNum(0);
  RECPacket->setTimestamp(0);
  RECPacket->setSSRC(0);

  char* rec_data = ((char*) RECPacket) + sizeof(RTP_Packet);

  for(unsigned short i = 0; i < len - sizeof(RTP_Packet); i++)
    rec_data[i] = 0;
}
