#ifndef SESSION_HH   // TD: SESSION_H -> SESSION_HH wg. QoS-Manager
#define SESSION_HH

#include <stdlib.h>                     // exit()
#include <string.h>                     // strncmp()
#include <fstream.h>                    // ifstream
#include <semaphore.h>                  // sem_t startup
#include <RTP/RTP_Packet.hh>            // RTP_Packet
#include <RTP/RTCP_Packet.hh>           // RTCP_Packet
#include <RTP/RTP_Session.hh>           // RTP_Session
#include <byte_order/byte_order.hh>     // reverse()


// TD: Include des QoSManager
#include "qosmanager.h"


#define FILE_SIGNATURE  "#PMESH-C"
#define FRAME_SIGNATURE "#PMESH-B"
#define RTP_MAX_PAYLOAD 1012
#define NUMBER_SIZE     4
#define MAX_PMINFOS     (unsigned int) (1)
#define MAX_VERTICES    (unsigned int) ((RTP_MAX_PAYLOAD - NUMBER_SIZE) / sizeof(Vertex))
#define MAX_TRIANGLES   (unsigned int) ((RTP_MAX_PAYLOAD - NUMBER_SIZE) / sizeof(Triangle))
#define MAX_SPLITS      (unsigned int) ((RTP_MAX_PAYLOAD - NUMBER_SIZE) / sizeof(Vertex_Split))

#define BASE_BPS    (2000 + 3 * (20 + 12 + 8 + 4))
#define EXT_BPS     (RTP_MAX_PAYLOAD + (20 + 12 + 8))
#define MAXIMUM_BPS ((30 * BASE_BPS) + ((20000 / 8) * EXT_BPS))
#define START_BPS   ((30 * BASE_BPS) + (1000 / 8) * EXT_BPS)

enum RTP_PlaybackModus
{
  RTP_MODUS_SIMPLE = 0,
  RTP_MODUS_LOOP   = 1
};

enum RTP_PayloadType
{
  RTP_PM_INFOS       = 0,
  RTP_BASE_VERTICES  = 1,
  RTP_BASE_TRIANGLES = 2,
  RTP_VERTEX_SPLITS  = 3,
  RTP_FEC_XOR        = 4
};

enum RTCP_APP_Type
{
  RTCP_APP_HANDSHAKE = 0,
  RTCP_APP_BPS       = 3,
  RTCP_APP_RPS       = 1,
  RTCP_APP_VSF       = 2,
  RTCP_APP_MODUS     = 4
};

enum BANDWIDTH_ADJ
{
  DECREASE = 0,
  HOLD     = 1,
  INCREASE = 2
};

enum NETWORK_STATE
{
  UNLOADED  = 0,
  LOADED    = 1,
  CONGESTED = 2
};

class Prog4D_Session : public RTP_Session, public ifstream,
                       virtual public Coral::QoSManagerInterface
                 
{
private:
  streampos position;                     // Position of frame in byte stream
  unsigned int frames;                    // Number of frames remaining in file
  unsigned int packets;                   // Packets needed/remaining for actual frame
  unsigned int pminfos;                   // PM_Infos      remaining for actual frame
  unsigned int vertices;                  // Vertices      remaining for actual frame
  unsigned int triangles;                 // Triangles     remaining for actual frame
  unsigned int vertex_splits;             // Vertex Splits remaining for actual frame
  unsigned int pminfo_count;              // PMInfos       sent for actual frame
  unsigned int vertex_count;              // Vertices      sent for actual frame
  unsigned int triangle_count;            // Triangles     sent for actual frame
  unsigned int vertex_split_count;        // Vertex splits sent for actual frame
  unsigned int max_pminfos;               // Maximum number of pminfos per package
  unsigned int max_vertices;              // Maximum number of vertices per package
  unsigned int max_triangles;             // Maximum number of triangles per package
  unsigned int max_splits;                // Maximum number of vertex splits per package
  unsigned int timestamp;                 // Timestamp for actual frame
  RTP_PayloadType ptype;                  // Payload Type for actual packet
  RTP_PlaybackModus modus;                // Playback Modus of this session
  unsigned long bw_min;                   // The minimum bandwidth in bits per second
  unsigned long bw_cur;                   // The current bandwidth in bits per second

  // TD: Gewnschte Bandbreite
  unsigned long bw_wanted;

  // TD: Vom Benutzer gesetzte Bandbreite
  unsigned long bw_user;  

  unsigned long bw_max;                   // The maximum bandwidth in bits per second
  unsigned long bw_unt;                   // The bandwidth of an unit in bits
  unsigned long bw_ms;                    // The bandwidth of the mesh in bits
  unsigned long bw_bm;                    // The bandwidth of the base mesh bits
  unsigned long bw_vs;                    // The bandwidth of the vertex splits
  float threshold_lower;                  // Lower threshold to determine network state
  float threshold_upper;                  // Upper threshold to determine network state
  float filter_parameter;                 // Parameter for low-pass filter
  unsigned short rate;                    // The sampling rate or frames per second
  unsigned short frac;                    // Fraction of vertex splits to be sent [0%;100%]
private:
  double tu;                              // The interval between two consecutive units
  double ti;                              // The interval between two consecutive packets
  Time   tc;                              // The actual time
  Time   tn;                              // The next scheduled transmission time of a packet
  Time   ts;                              // The time to wait until next transmission time of a packet
private:
  sem_t startup;
public:
  // TD: QoSManager als Parameter fr Konstruktor
  Prog4D_Session(Coral::QoSManager* qosManager);
private:
  void init();
  void next();
  void adapt(float);
  void sendLoopRTP();
  void recvLoopRTP();
  void calcTransInterval();
  void feedback(RTCP_Packet*);
  RTP_PlaybackModus getModus();
  unsigned long getBPS();
  unsigned short getRate();
  unsigned short getFraction();
  void setModus(RTP_PlaybackModus);
  void setBPS(unsigned long);
  void setRate(unsigned short);
  void setFraction(unsigned short);

   // TD: Attribut fr QoS-Manager, neue Methoden
   private:
   Coral::QoSManager* QoSMgr;
   cardinal              StreamIdentifier;
   card8                 TrafficClass;
   card8                 ExtensionTrafficClass;

   public:
   void adaptTransportInfo(card32 flowIdentifier, Coral::ExtendedTransportInfo ti);
   void getTransportInfo(Coral::TransportInfo* transportInfo);
   private:
   void managementUpdate();
   void getTransportInfoConstants(Coral::TransportInfo* transportInfo);
   // TD: Ende -------------------
};


class PMInfo
{
public:
  unsigned int base_vertices;           // Number of base vertices
  unsigned int base_triangles;          // Number of base triangles
  unsigned int vertex_splits;           // Number of vertex splits
};

class Vertex
{
public:
  float x;                              // x coordinate
  float y;                              // y coordinate
  float z;                              // z coordinate
};

class Triangle
{
public:
  unsigned int a;                       // Reference to number of base vertice
  unsigned int b;                       // Reference to number of base vertice
  unsigned int c;                       // Reference to number of base vertice
};

class Vertex_Split
{
public:
  unsigned int a;                       // Reference to number of base vertice
  unsigned int b;                       // Reference to number of base vertice
  unsigned int c;                       // Reference to number of base vertice
  Vertex v;                             // Vertex to add into progressive mesh
};

istream& operator>>(istream&, PMInfo&);
ostream& operator<<(ostream&, PMInfo&);

istream& operator>>(istream&, Vertex&);
ostream& operator<<(ostream&, Vertex&);

istream& operator>>(istream&, Triangle&);
ostream& operator<<(ostream&, Triangle&);

istream& operator>>(istream&, Vertex_Split&);
ostream& operator<<(ostream&, Vertex_Split&);

class Progressive_Packet : public RTP_Packet
{
public:
  u_int32 number;
  union
  {
    PMInfo pminfo[MAX_PMINFOS];
    Vertex vertex[MAX_VERTICES];
    Triangle triangle[MAX_TRIANGLES];
    Vertex_Split split[MAX_SPLITS];
  };
protected:
  unsigned short length;
public:
  u_int32 getNumber();
  void setNumber(u_int32);
  unsigned short getLength();
  void setLength(unsigned short);
};

#endif
