// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### QInfoTabWidget Implementation                                    ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "qinfotabwidget.h"


#include <qapp.h>
#if QT_VERSION < 200
#error QT Version 2.00 or better required!!!
#endif
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qdict.h>
#include <qlist.h>


namespace Coral {


// ###### Constructor #######################################################
QInfoWidget::QInfoWidget(const InfoTable* table,
                         QWidget*         parent,
                         const char*      name)
   : QWidget(parent,name)
{
   Table = table;

   // ====== Initialize layout and QWhatsThis ===============================
   QGridLayout* layout = new QGridLayout(this,Table->Entries,10);
   CHECK_PTR(layout);
   layout->setColStretch(1,10);

   WhatsThis = new QWhatsThis(this);
   CHECK_PTR(WhatsThis);

   // ====== Create info table ==============================================
   for(cardinal i = 0;i < Table->Entries;i++) {
      QLabel* label = new QLabel(QString(Table->Entry[i].Title) + ": ",this);
      CHECK_PTR(label);
      label->setMinimumSize(label->sizeHint());
      layout->addWidget(label,i,0);
      WhatsThis->add(label,Table->Entry[i].Help);

      QLabel* value = new QLabel("--- N/A ---",this);
      CHECK_PTR(value);
      value->setMinimumSize(value->sizeHint());
      layout->addWidget(value,i,1);
      WhatsThis->add(value,Table->Entry[i].Help);

      LabelDict.insert(Table->Entry[i].ID,value);
      LabelList.append(value);
   }
}


// ###### Update ############################################################
bool QInfoWidget::update(const QString& id, const QString& value)
{
   QLabel* label = LabelDict.find(id);
   if(label != NULL) {
      label->setText(value);
      return(true);
   }
   return(false);
}


// ###### Clear all entries #################################################
void QInfoWidget::clear()
{
   QLabel* label = LabelList.first();
   while(label != NULL) {
      label->setText("N/A");
      label = LabelList.next();
   }
}


// ###### Constructor #######################################################
QInfoTabWidget::QInfoTabWidget(const InfoTable* table,
                               const char*      title,
                               const char*      pixmapName,
                               QWidget*         parent = NULL,
                               const char*      name   = NULL)
   : QTabWidget(parent,name)
{
   addTable(table,title,pixmapName);
}


// ###### Add InfoTable #####################################################
QInfoWidget* QInfoTabWidget::addTable(const InfoTable* table,
                                      const char*      title,
                                      const char*      pixmapName)
{
   QInfoWidget* infoWidget = new QInfoWidget(table,this);
   CHECK_PTR(infoWidget);
   addTab(infoWidget,QPixmap(pixmapName),title);
   InfoWidgetList.append(infoWidget);
   return(infoWidget);
}


// ###### Update ############################################################
bool QInfoTabWidget::update(const QString& id, const QString& value)
{
   cardinal count = 0;

   QInfoWidget* infoWidget = InfoWidgetList.first();
   while(infoWidget != NULL) {
      if(infoWidget->update(id,value)) {
         count++;
      }
      infoWidget = InfoWidgetList.next();
   }
   if(count == 0) {
      cerr << "QInfoTabWidget::update() - Unknown ID " << id << "!" << endl;
      return(false);
   }
   return(true);
}


// ###### Clear all entries #################################################
void QInfoTabWidget::clear()
{
   QInfoWidget* infoWidget = InfoWidgetList.first();
   while(infoWidget != NULL) {
      infoWidget->clear();
      infoWidget = InfoWidgetList.next();
   }
}


}


#include "qinfotabwidget.moc"
