// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### RTCP Packet Implementation                                       ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "rtcppacket.h"


namespace Coral {


// ##########################################################################
// #### RTCP Common Header                                               ####
// ##########################################################################


// ###### Contructor ########################################################
RTCPCommonHeader::RTCPCommonHeader()
{ }


// ##########################################################################
// #### RTCP Report                                                      ####
// ##########################################################################


// ###### Contructor ########################################################
RTCPReport::RTCPReport()
{ }


// ##########################################################################
// #### RTCP Sender Report                                               ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPSenderReport::RTCPSenderReport() { }
RTCPSenderReport::RTCPSenderReport(const card32 syncSource,
                                   const card8 count)
{
   init(syncSource,count);
}


// ###### Initialize ########################################################
void RTCPSenderReport::init(const card32 syncSource, const card8 count)
{
   V    = RTPConstants::RTPVersion;
   P    = false;
   C    = count;
   PT   = RTCP_SR;
   setLength(sizeof(*this));
   setSSRC(syncSource);
}


// ##########################################################################
// #### RTCP Receiver Report                                             ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPReceiverReport::RTCPReceiverReport() { }
RTCPReceiverReport::RTCPReceiverReport(const card32 syncSource,
                                       const card8 count)
{ 
   init(syncSource,count);
}


// ###### Initialize ########################################################
void RTCPReceiverReport::init(const card32 syncSource, const card8 count)
{
  V    = RTPConstants::RTPVersion;
  P    = false;
  C    = count;
  PT   = RTCP_RR;
  setLength(sizeof(*this));
  setSSRC(syncSource);
}


// ##########################################################################
// #### RTCP Sender Info Block                                           ####
// ##########################################################################


// ###### Contructor ########################################################
RTCPSenderInfoBlock::RTCPSenderInfoBlock()
{ }


// ##########################################################################
// #### RTCP Reception Report Block                                      ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPReceptionReportBlock::RTCPReceptionReportBlock() { }
RTCPReceptionReportBlock::RTCPReceptionReportBlock(const card32 ssrc)
{
   init(ssrc);
}


// ###### Initialize ########################################################
void RTCPReceptionReportBlock::init(const card32 ssrc)
{
   SSRC = translate32(ssrc);
}



// ##########################################################################
// #### RTCP Source Description                                          ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPSourceDescription::RTCPSourceDescription() { }
RTCPSourceDescription::RTCPSourceDescription(const card8 count)
{ 
   init(count);
}


// ###### Initialize ########################################################
void RTCPSourceDescription::init(const card8 count)
{
  V  = RTPConstants::RTPVersion;
  P  = false;
  C  = count;
  PT = RTCP_SDES;
  setLength(sizeof(*this));
}


// ##########################################################################
// #### RTCP Bye                                                         ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPBye::RTCPBye() { }
RTCPBye::RTCPBye(const card8 count)
{
   init(count);
}


// ###### Initialize ########################################################
void RTCPBye::init(const card8 count)
{
  V  = RTPConstants::RTPVersion;
  P  = false;
  C  = count;
  PT = RTCP_BYE;
  setLength(sizeof(*this));
}


// ##########################################################################
// #### RTCP App                                                         ####
// ##########################################################################


// ###### Contructors #######################################################
RTCPApp::RTCPApp() { }
RTCPApp::RTCPApp(const card8 subtype)
{ 
   init(subtype);
}


// ###### Initialize ########################################################
void RTCPApp::init(const card8 subtype)
{
  V  = RTPConstants::RTPVersion;
  P  = false;
  C  = subtype;
  PT = RTCP_APP;
  setLength(sizeof(*this));
}


}
