// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Stream Source and Destination Implementation                     ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "streamsrcdest.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
StreamSrcDest::StreamSrcDest()
{
   reset();
}


// ###### Reset #############################################################
void StreamSrcDest::reset()
{
   Source.reset();
   Destination.reset();
   FlowLabel    = 0;
   TrafficClass = 0;
   IsValid      = false;
   pad          = 0;
}


// ###### Translate byte order ##############################################
void StreamSrcDest::translate()
{
   FlowLabel = translate32(FlowLabel);
}


// ###### Comparision operator ##############################################
int StreamSrcDest::operator==(const StreamSrcDest& ssd) const
{
   return((IsValid     == ssd.IsValid)     &&
          (Source      == ssd.Source)      &&
          (Destination == ssd.Destination) &&
          (FlowLabel   == ssd.FlowLabel));
}


// ###### Comparision operator ##############################################
int StreamSrcDest::operator!=(const StreamSrcDest& ssd) const
{
   return((IsValid     != ssd.IsValid)     ||
          (Source      != ssd.Source)      ||
          (Destination != ssd.Destination) ||
          (FlowLabel   != ssd.FlowLabel));
}


// ###### Output operator ###################################################
ostream& operator<<(ostream& os, const StreamSrcDest& ssd)
{
   if(ssd.IsValid) {
      os << "   Source              = " << InternetAddress(ssd.Source) << endl;
      os << "   Destination         = " << InternetAddress(ssd.Destination) << endl;
      char str[64];
      snprintf((char*)&str,sizeof(str),"$%02x",ssd.TrafficClass);
      os << "   Traffic Class       = " << str << endl;
      snprintf((char*)&str,sizeof(str),"$%05x",ssd.FlowLabel);
      os << "   Flow Label          = " << str << endl;
   }
   else {
      os << "   (not valid)" << endl;
   }
   return(os);
}


}
